@@ -5,12 +5,11 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
-{-# LANGUAGE GeneralizedNewtypeDeriving, PackageImports #-}
+{-# LANGUAGE CPP, GeneralizedNewtypeDeriving, PackageImports #-}
 
 module Annex (
 	Annex,
 	AnnexState(..),
-	PreferredContentMap,
 	new,
 	run,
 	eval,
@@ -29,14 +28,10 @@ module Annex (
 	getGitConfig,
 	changeGitConfig,
 	changeGitRepo,
+	getRemoteGitConfig,
 	withCurrentState,
 ) where
 
-import "mtl" Control.Monad.Reader
-import "MonadCatchIO-transformers" Control.Monad.CatchIO
-import System.Posix.Types (Fd)
-import Control.Concurrent
-
 import Common
 import qualified Git
 import qualified Git.Config
@@ -45,7 +40,9 @@ import Git.CatFile
 import Git.CheckAttr
 import Git.CheckIgnore
 import Git.SharedRepository
+import qualified Git.Hook
 import qualified Git.Queue
+import Types.Key
 import Types.Backend
 import Types.GitConfig
 import qualified Types.Remote
@@ -56,7 +53,19 @@ import Types.Group
 import Types.Messages
 import Types.UUID
 import Types.FileMatcher
-import qualified Utility.Matcher
+import Types.NumCopies
+import Types.LockPool
+import Types.MetaData
+import Types.DesktopNotify
+import Types.CleanupActions
+#ifdef WITH_QUVI
+import Utility.Quvi (QuviVersion)
+#endif
+import Utility.InodeCache
+
+import "mtl" Control.Monad.Reader
+import Control.Monad.Catch
+import Control.Concurrent
 import qualified Data.Map as M
 import qualified Data.Set as S
 
@@ -69,20 +78,22 @@ newtype Annex a = Annex { runAnnex :: ReaderT (MVar AnnexState) IO a }
 		Monad,
 		MonadIO,
 		MonadReader (MVar AnnexState),
-		MonadCatchIO,
+		MonadCatch,
+		MonadThrow,
+#if MIN_VERSION_exceptions(0,6,0)
+		MonadMask,
+#endif
 		Functor,
 		Applicative
 	)
 
-type Matcher a = Either [Utility.Matcher.Token a] (Utility.Matcher.Matcher a)
-type PreferredContentMap = M.Map UUID (Utility.Matcher.Matcher (S.Set UUID -> FileInfo -> Annex Bool))
-
 -- internal state storage
 data AnnexState = AnnexState
 	{ repo :: Git.Repo
 	, gitconfig :: GitConfig
 	, backends :: [BackendA Annex]
 	, remotes :: [Types.Remote.RemoteA Annex]
+	, remoteannexstate :: M.Map UUID AnnexState
 	, output :: MessageState
 	, force :: Bool
 	, fast :: Bool
@@ -94,21 +105,31 @@ data AnnexState = AnnexState
 	, checkattrhandle :: Maybe CheckAttrHandle
 	, checkignorehandle :: Maybe (Maybe CheckIgnoreHandle)
 	, forcebackend :: Maybe String
-	, forcenumcopies :: Maybe Int
-	, limit :: Matcher (FileInfo -> Annex Bool)
+	, globalnumcopies :: Maybe NumCopies
+	, forcenumcopies :: Maybe NumCopies
+	, limit :: ExpandableMatcher Annex
 	, uuidmap :: Maybe UUIDMap
-	, preferredcontentmap :: Maybe PreferredContentMap
+	, preferredcontentmap :: Maybe (FileMatcherMap Annex)
+	, requiredcontentmap :: Maybe (FileMatcherMap Annex)
 	, shared :: Maybe SharedRepository
 	, forcetrust :: TrustMap
 	, trustmap :: Maybe TrustMap
 	, groupmap :: Maybe GroupMap
 	, ciphers :: M.Map StorableCipher Cipher
-	, lockpool :: M.Map FilePath Fd
+	, lockpool :: LockPool
 	, flags :: M.Map String Bool
 	, fields :: M.Map String String
-	, cleanup :: M.Map String (Annex ())
-	, inodeschanged :: Maybe Bool
+	, modmeta :: [ModMeta]
+	, cleanup :: M.Map CleanupAction (Annex ())
+	, sentinalstatus :: Maybe SentinalStatus
 	, useragent :: Maybe String
+	, errcounter :: Integer
+	, unusedkeys :: Maybe (S.Set Key)
+#ifdef WITH_QUVI
+	, quviversion :: Maybe QuviVersion
+#endif
+	, existinghooks :: M.Map Git.Hook.Hook Bool
+	, desktopnotify :: DesktopNotify
 	}
 
 newState :: GitConfig -> Git.Repo -> AnnexState
@@ -117,6 +138,7 @@ newState c r = AnnexState
 	, gitconfig = c
 	, backends = []
 	, remotes = []
+	, remoteannexstate = M.empty
 	, output = defaultMessageState
 	, force = False
 	, fast = False
@@ -128,10 +150,12 @@ newState c r = AnnexState
 	, checkattrhandle = Nothing
 	, checkignorehandle = Nothing
 	, forcebackend = Nothing
+	, globalnumcopies = Nothing
 	, forcenumcopies = Nothing
-	, limit = Left []
+	, limit = BuildingMatcher []
 	, uuidmap = Nothing
 	, preferredcontentmap = Nothing
+	, requiredcontentmap = Nothing
 	, shared = Nothing
 	, forcetrust = M.empty
 	, trustmap = Nothing
@@ -140,9 +164,17 @@ newState c r = AnnexState
 	, lockpool = M.empty
 	, flags = M.empty
 	, fields = M.empty
+	, modmeta = []
 	, cleanup = M.empty
-	, inodeschanged = Nothing
+	, sentinalstatus = Nothing
 	, useragent = Nothing
+	, errcounter = 0
+	, unusedkeys = Nothing
+#ifdef WITH_QUVI
+	, quviversion = Nothing
+#endif
+	, existinghooks = M.empty
+	, desktopnotify = mempty
 	}
 
 {- Makes an Annex state object for the specified git repo.
@@ -191,9 +223,9 @@ setField field value = changeState $ \s ->
 	s { fields = M.insertWith' const field value $ fields s }
 
 {- Adds a cleanup action to perform. -}
-addCleanup :: String -> Annex () -> Annex ()
-addCleanup uid a = changeState $ \s ->
-	s { cleanup = M.insertWith' const uid a $ cleanup s }
+addCleanup :: CleanupAction -> Annex () -> Annex ()
+addCleanup k a = changeState $ \s ->
+	s { cleanup = M.insertWith' const k a $ cleanup s }
 
 {- Sets the type of output to emit. -}
 setOutput :: OutputType -> Annex ()
@@ -241,6 +273,13 @@ changeGitRepo r = changeState $ \s -> s
 	, gitconfig = extractGitConfig r
 	}
 
+{- Gets the RemoteGitConfig from a remote, given the Git.Repo for that
+ - remote. -}
+getRemoteGitConfig :: Git.Repo -> Annex RemoteGitConfig
+getRemoteGitConfig r = do
+	g <- gitRepo
+	return $ extractRemoteGitConfig g (Git.repoDescribe r)
+
 {- Converts an Annex action into an IO action, that runs with a copy
  - of the current Annex state. 
  -
