@@ -14,13 +14,14 @@
 
 module Annex.Exception (
 	bracketIO,
+	bracketAnnex,
 	tryAnnex,
 	tryAnnexIO,
 	throwAnnex,
 	catchAnnex,
 ) where
 
-import qualified "MonadCatchIO-transformers" Control.Monad.CatchIO as M
+import qualified Control.Monad.Catch as M
 import Control.Exception
 
 import Common.Annex
@@ -29,6 +30,9 @@ import Common.Annex
 bracketIO :: IO v -> (v -> IO b) -> (v -> Annex a) -> Annex a
 bracketIO setup cleanup = M.bracket (liftIO setup) (liftIO . cleanup)
 
+bracketAnnex :: Annex v -> (v -> Annex b) -> (v -> Annex a) -> Annex a
+bracketAnnex = M.bracket
+
 {- try in the Annex monad -}
 tryAnnex :: Annex a -> Annex (Either SomeException a)
 tryAnnex = M.try
@@ -39,7 +43,7 @@ tryAnnexIO = M.try
 
 {- throw in the Annex monad -}
 throwAnnex :: Exception e => e -> Annex a
-throwAnnex = M.throw
+throwAnnex = M.throwM
 
 {- catch in the Annex monad -}
 catchAnnex :: Exception e => Annex a -> (e -> Annex a) -> Annex a
