@@ -13,12 +13,11 @@
 
 module Annex.Journal where
 
-import System.IO.Binary
-
 import Common.Annex
 import Annex.Exception
 import qualified Git
 import Annex.Perms
+import Annex.LockFile
 
 {- Records content for a file in the branch to the journal.
  -
@@ -31,14 +30,19 @@ import Annex.Perms
  -}
 setJournalFile :: JournalLocked -> FilePath -> String -> Annex ()
 setJournalFile _jl file content = do
+	tmp <- fromRepo gitAnnexTmpMiscDir
 	createAnnexDirectory =<< fromRepo gitAnnexJournalDir
-	createAnnexDirectory =<< fromRepo gitAnnexTmpDir
+	createAnnexDirectory tmp
 	-- journal file is written atomically
 	jfile <- fromRepo $ journalFile file
-	tmp <- fromRepo gitAnnexTmpDir
 	let tmpfile = tmp </> takeFileName jfile
 	liftIO $ do
-		writeBinaryFile tmpfile content
+		withFile tmpfile WriteMode $ \h -> do
+			fileEncoding h
+#ifdef mingw32_HOST_OS
+			hSetNewlineMode h noNewlineTranslation
+#endif
+			hPutStr h content
 		moveFile tmpfile jfile
 
 {- Gets any journalled content for a file in the branch. -}
@@ -50,7 +54,7 @@ getJournalFile _jl = getJournalFileStale
  - changes. -}
 getJournalFileStale :: FilePath -> Annex (Maybe String)
 getJournalFileStale file = inRepo $ \g -> catchMaybeIO $
-	readFileStrict $ journalFile file g
+	readFileStrictAnyEncoding $ journalFile file g
 
 {- List of files that have updated content in the journal. -}
 getJournalledFiles :: JournalLocked -> Annex [FilePath]
@@ -73,9 +77,18 @@ getJournalFilesStale = do
 		getDirectoryContents $ gitAnnexJournalDir g
 	return $ filter (`notElem` [".", ".."]) fs
 
+withJournalHandle :: (DirectoryHandle -> IO a) -> Annex a
+withJournalHandle a = do
+	d <- fromRepo gitAnnexJournalDir
+	bracketIO (openDirectory d) closeDirectory (liftIO . a)
+
 {- Checks if there are changes in the journal. -}
 journalDirty :: Annex Bool
-journalDirty = not . null <$> getJournalFilesStale
+journalDirty = do
+	d <- fromRepo gitAnnexJournalDir
+	liftIO $ 
+		(not <$> isDirectoryEmpty d)
+			`catchIO` (const $ doesDirectoryExist d)
 
 {- Produces a filename to use in the journal for a file on the branch.
  -
@@ -105,24 +118,4 @@ data JournalLocked = ProduceJournalLocked
 {- Runs an action that modifies the journal, using locking to avoid
  - contention with other git-annex processes. -}
 lockJournal :: (JournalLocked -> Annex a) -> Annex a
-lockJournal a = do
-	lockfile <- fromRepo gitAnnexJournalLock
-	createAnnexDirectory $ takeDirectory lockfile
-	mode <- annexFileMode
-	bracketIO (lock lockfile mode) unlock (const $ a ProduceJournalLocked)
-  where
-#ifndef mingw32_HOST_OS
-	lock lockfile mode = do
-		l <- noUmask mode $ createFile lockfile mode
-		waitToSetLock l (WriteLock, AbsoluteSeek, 0, 0)
-		return l
-#else
-	lock lockfile _mode = do
-		writeFile lockfile ""
-		return lockfile
-#endif
-#ifndef mingw32_HOST_OS
-	unlock = closeFd
-#else
-	unlock = removeFile
-#endif
+lockJournal a = withExclusiveLock gitAnnexJournalLock $ a ProduceJournalLocked
