@@ -21,6 +21,7 @@ import Assistant.Threads.Pusher
 import Assistant.Threads.Merger
 import Assistant.Threads.TransferWatcher
 import Assistant.Threads.Transferrer
+import Assistant.Threads.RemoteControl
 import Assistant.Threads.SanityChecker
 import Assistant.Threads.Cronner
 import Assistant.Threads.ProblemFixer
@@ -45,15 +46,19 @@ import Assistant.Threads.XMPPClient
 import Assistant.Threads.XMPPPusher
 #endif
 #else
-#warning Building without the webapp. You probably need to install Yesod..
 import Assistant.Types.UrlRenderer
 #endif
 import qualified Utility.Daemon
-import Utility.LogFile
 import Utility.ThreadScheduler
 import Utility.HumanTime
-import Annex.Perms
 import qualified Build.SysConfig as SysConfig
+import Annex.Perms
+import Utility.LogFile
+#ifdef mingw32_HOST_OS
+import Utility.Env
+import Config.Files
+import System.Environment (getArgs)
+#endif
 
 import System.Log.Logger
 import Network.Socket (HostName)
@@ -71,20 +76,17 @@ startDaemon assistant foreground startdelay cannotrun listenhost startbrowser =
 	Annex.changeState $ \s -> s { Annex.daemon = True }
 	pidfile <- fromRepo gitAnnexPidFile
 	logfile <- fromRepo gitAnnexLogFile
+	liftIO $ debugM desc $ "logging to " ++ logfile
 #ifndef mingw32_HOST_OS
 	createAnnexDirectory (parentDir logfile)
-	logfd <- liftIO $ openLog logfile
+	logfd <- liftIO $ handleToFd =<< openLog logfile
 	if foreground
 		then do
 			origout <- liftIO $ catchMaybeIO $ 
 				fdToHandle =<< dup stdOutput
 			origerr <- liftIO $ catchMaybeIO $ 
 				fdToHandle =<< dup stdError
-			let undaemonize a = do
-				debugM desc $ "logging to " ++ logfile
-				Utility.Daemon.lockPidFile pidfile
-				Utility.LogFile.redirLog logfd
-				a
+			let undaemonize = Utility.Daemon.foreground logfd (Just pidfile)
 			start undaemonize $ 
 				case startbrowser of
 					Nothing -> Nothing
@@ -92,11 +94,29 @@ startDaemon assistant foreground startdelay cannotrun listenhost startbrowser =
 		else
 			start (Utility.Daemon.daemonize logfd (Just pidfile) False) Nothing
 #else
-	-- Windows is always foreground, and has no log file.
-	start id $
-		case startbrowser of
-			Nothing -> Nothing
-			Just a -> Just $ a Nothing Nothing
+	-- Windows doesn't daemonize, but does redirect output to the
+	-- log file. The only way to do so is to restart the program.
+	when (foreground || not foreground) $ do
+		let flag = "GIT_ANNEX_OUTPUT_REDIR"
+		createAnnexDirectory (parentDir logfile)
+		ifM (liftIO $ isNothing <$> getEnv flag)
+			( liftIO $ withFile devNull WriteMode $ \nullh -> do
+				loghandle <- openLog logfile
+				e <- getEnvironment
+				cmd <- readProgramFile
+				ps <- getArgs
+				(_, _, _, pid) <- createProcess (proc cmd ps)
+					{ env = Just (addEntry flag "1" e)
+					, std_in = UseHandle nullh
+					, std_out = UseHandle loghandle
+					, std_err = UseHandle loghandle
+					}
+				exitWith =<< waitForProcess pid
+			, start (Utility.Daemon.foreground (Just pidfile)) $
+				case startbrowser of
+					Nothing -> Nothing
+					Just a -> Just $ a Nothing Nothing
+			)
 #endif
   where
   	desc
@@ -120,7 +140,7 @@ startDaemon assistant foreground startdelay cannotrun listenhost startbrowser =
 		notice ["starting", desc, "version", SysConfig.packageversion]
 		urlrenderer <- liftIO newUrlRenderer
 #ifdef WITH_WEBAPP
-		let webappthread = [ assist $ webAppThread d urlrenderer False cannotrun listenhost Nothing webappwaiter ]
+		let webappthread = [ assist $ webAppThread d urlrenderer False cannotrun Nothing listenhost webappwaiter ]
 #else
 		let webappthread = []
 #endif
@@ -144,8 +164,9 @@ startDaemon assistant foreground startdelay cannotrun listenhost startbrowser =
 				, assist $ transferWatcherThread
 				, assist $ transferPollerThread
 				, assist $ transfererThread
+				, assist $ remoteControlThread
 				, assist $ daemonStatusThread
-				, assist $ sanityCheckerDailyThread
+				, assist $ sanityCheckerDailyThread urlrenderer
 				, assist $ sanityCheckerHourlyThread
 				, assist $ problemFixerThread urlrenderer
 #ifdef WITH_CLIBS
