@@ -35,6 +35,7 @@ import qualified Annex
 import Utility.InodeCache
 import Annex.Content.Direct
 import qualified Command.Sync
+import qualified Git.Branch
 
 import Data.Time.Clock
 import Data.Tuple.Utils
@@ -219,7 +220,11 @@ commitStaged = do
 	v <- tryAnnex Annex.Queue.flush
 	case v of
 		Left _ -> return False
-		Right _ -> Command.Sync.commitStaged ""
+		Right _ -> do
+			ok <- Command.Sync.commitStaged Git.Branch.AutomaticCommit ""
+			when ok $
+				Command.Sync.updateSyncBranch =<< inRepo Git.Branch.current
+			return ok
 
 {- OSX needs a short delay after a file is added before locking it down,
  - when using a non-direct mode repository, as pasting a file seems to
@@ -313,10 +318,11 @@ handleAdds havelsof delayadd cs = returnWhen (null incomplete) $ do
 	adddirect toadd = do
 		ct <- liftAnnex compareInodeCachesWith
 		m <- liftAnnex $ removedKeysMap ct cs
+		delta <- liftAnnex getTSDelta
 		if M.null m
 			then forM toadd add
 			else forM toadd $ \c -> do
-				mcache <- liftIO $ genInodeCache $ changeFile c
+				mcache <- liftIO $ genInodeCache (changeFile c) delta
 				case mcache of
 					Nothing -> add c
 					Just cache ->
@@ -448,7 +454,7 @@ safeToAdd havelsof delayadd pending inprocess = do
 			let segments = segmentXargs $ map keyFilename keysources
 			concat <$> forM segments (\fs -> Lsof.query $ "--" : fs)
 		, do
-			tmpdir <- fromRepo gitAnnexTmpDir
+			tmpdir <- fromRepo gitAnnexTmpMiscDir
 			liftIO $ Lsof.queryDir tmpdir
 		)
 
@@ -464,7 +470,7 @@ checkChangeContent change@(Change { changeInfo = i }) =
 		Nothing -> noop
 		Just k -> whenM (scanComplete <$> getDaemonStatus) $ do
 			present <- liftAnnex $ inAnnex k
-			if present
+			void $ if present
 				then queueTransfers "new file created" Next k (Just f) Upload
 				else queueTransfers "new or renamed file wanted" Next k (Just f) Download
 			handleDrops "file renamed" present k (Just f) Nothing
