@@ -42,17 +42,20 @@ xmppClientThread urlrenderer = namedThread "XMPPClient" $
 	restartableClient . xmppClient urlrenderer =<< getAssistant id
 
 {- Runs the client, handing restart events. -}
-restartableClient :: (XMPPCreds -> IO ()) -> Assistant ()
+restartableClient :: (XMPPCreds -> UUID -> IO ()) -> Assistant ()
 restartableClient a = forever $ go =<< liftAnnex getXMPPCreds
   where
 	go Nothing = waitNetMessagerRestart
 	go (Just creds) = do
-		tid <- liftIO $ forkIO $ a creds
+		xmppuuid <- maybe NoUUID Remote.uuid . headMaybe 
+			. filter Remote.isXMPPRemote . syncRemotes
+			<$> getDaemonStatus
+		tid <- liftIO $ forkIO $ a creds xmppuuid
 		waitNetMessagerRestart
 		liftIO $ killThread tid
 
-xmppClient :: UrlRenderer -> AssistantData -> XMPPCreds -> IO ()
-xmppClient urlrenderer d creds =
+xmppClient :: UrlRenderer -> AssistantData -> XMPPCreds -> UUID -> IO ()
+xmppClient urlrenderer d creds xmppuuid =
 	retry (runclient creds) =<< getCurrentTime
   where
 	liftAssistant = runAssistant d
@@ -68,8 +71,11 @@ xmppClient urlrenderer d creds =
 		liftAssistant $
 			updateBuddyList (const noBuddies) <<~ buddyList
 		void client
-		liftAssistant $ modifyDaemonStatus_ $ \s -> s
-			{ xmppClientID = Nothing }
+		liftAssistant $ do
+			modifyDaemonStatus_ $ \s -> s
+				{ xmppClientID = Nothing }
+			changeCurrentlyConnected $ S.delete xmppuuid
+			
 		now <- getCurrentTime
 		if diffUTCTime now starttime > 300
 			then do
@@ -87,6 +93,7 @@ xmppClient urlrenderer d creds =
 		inAssistant $ do
 			modifyDaemonStatus_ $ \s -> s
 				{ xmppClientID = Just $ xmppJID creds }
+			changeCurrentlyConnected $ S.insert xmppuuid
 			debug ["connected", logJid selfjid]
 
 		lasttraffic <- liftIO $ atomically . newTMVar =<< getCurrentTime
@@ -322,7 +329,7 @@ pairMsgReceived urlrenderer PairReq theiruuid selfjid theirjid
 	| baseJID selfjid == baseJID theirjid = autoaccept
 	| otherwise = do
 		knownjids <- mapMaybe (parseJID . getXMPPClientID)
-			. filter isXMPPRemote . syncRemotes <$> getDaemonStatus
+			. filter Remote.isXMPPRemote . syncRemotes <$> getDaemonStatus
 		um <- liftAnnex uuidMap
 		if elem (baseJID theirjid) knownjids && M.member theiruuid um
 			then autoaccept
