@@ -11,11 +11,12 @@ module Assistant.WebApp.Configurators.Edit where
 
 import Assistant.WebApp.Common
 import Assistant.WebApp.Gpg
+import Assistant.WebApp.Configurators
 import Assistant.DaemonStatus
 import Assistant.WebApp.MakeRemote (uniqueRemoteName)
-import Assistant.WebApp.Configurators.XMPP (xmppNeeded)
 import Assistant.ScanRemotes
 import Assistant.Sync
+import Assistant.Alert
 import qualified Assistant.WebApp.Configurators.AWS as AWS
 import qualified Assistant.WebApp.Configurators.IA as IA
 #ifdef WITH_S3
@@ -30,6 +31,7 @@ import Logs.PreferredContent
 import Logs.Remote
 import Types.StandardGroups
 import qualified Git
+import qualified Git.Types as Git
 import qualified Git.Command
 import qualified Git.Config
 import qualified Annex
@@ -59,6 +61,10 @@ data RepoConfig = RepoConfig
 
 getRepoConfig :: UUID -> Maybe Remote -> Annex RepoConfig
 getRepoConfig uuid mremote = do
+	-- Ensure we're editing current data by discarding caches.
+	void groupMapLoad
+	void uuidMapLoad
+
 	groups <- lookupGroups uuid
 	remoteconfig <- M.lookup uuid <$> readRemoteLog
 	let (repogroup, associateddirectory) = case getStandardGroup groups of
@@ -137,19 +143,25 @@ setRepoConfig uuid mremote oldc newc = do
 
 	legalName = makeLegalName . T.unpack . repoName
 
-editRepositoryAForm :: Bool -> RepoConfig -> MkAForm RepoConfig
-editRepositoryAForm ishere def = RepoConfig
+editRepositoryAForm :: Maybe Remote -> RepoConfig -> MkAForm RepoConfig
+editRepositoryAForm mremote def = RepoConfig
 	<$> areq (if ishere then readonlyTextField else textField)
-		"Name" (Just $ repoName def)
-	<*> aopt textField "Description" (Just $ repoDescription def)
-	<*> areq (selectFieldList groups `withNote` help) "Repository group" (Just $ repoGroup def)
+		(bfs "Name") (Just $ repoName def)
+	<*> aopt textField (bfs "Description") (Just $ repoDescription def)
+	<*> areq (selectFieldList groups `withNote` help) (bfs "Repository group") (Just $ repoGroup def)
 	<*> associateddirectory
 	<*> areq checkBoxField "Syncing enabled" (Just $ repoSyncable def)
   where
+	ishere = isNothing mremote
+	isspecial = fromMaybe False $
+		(== Git.Unknown) . Git.location . Remote.repo <$> mremote
 	groups = customgroups ++ standardgroups
 	standardgroups :: [(Text, RepoGroup)]
-	standardgroups = map (\g -> (T.pack $ descStandardGroup g , RepoGroupStandard g))
-		[minBound :: StandardGroup .. maxBound :: StandardGroup]
+	standardgroups = map (\g -> (T.pack $ descStandardGroup g , RepoGroupStandard g)) $
+		filter sanegroup [minBound..maxBound]
+	sanegroup
+		| isspecial = const True
+		| otherwise = not . specialRemoteOnly
 	customgroups :: [(Text, RepoGroup)]
 	customgroups = case repoGroup def of
 		RepoGroupCustom s -> [(T.pack s, RepoGroupCustom s)]
@@ -158,7 +170,7 @@ editRepositoryAForm ishere def = RepoConfig
 
 	associateddirectory = case repoAssociatedDirectory def of
 		Nothing -> aopt hiddenField "" Nothing
-		Just d -> aopt textField "Associated directory" (Just $ Just d)
+		Just d -> aopt textField (bfs "Associated directory") (Just $ Just d)
 
 getEditRepositoryR :: RepoId -> Handler Html
 getEditRepositoryR = postEditRepositoryR
@@ -176,7 +188,7 @@ getEditNewCloudRepositoryR :: UUID -> Handler Html
 getEditNewCloudRepositoryR = postEditNewCloudRepositoryR
 
 postEditNewCloudRepositoryR :: UUID -> Handler Html
-postEditNewCloudRepositoryR uuid = xmppNeeded >> editForm True (RepoUUID uuid)
+postEditNewCloudRepositoryR uuid = connectionNeeded >> editForm True (RepoUUID uuid)
 
 editForm :: Bool -> RepoId -> Handler Html
 editForm new (RepoUUID uuid) = page "Edit repository" (Just Configuration) $ do
@@ -187,7 +199,7 @@ editForm new (RepoUUID uuid) = page "Edit repository" (Just Configuration) $ do
 	curr <- liftAnnex $ getRepoConfig uuid mremote
 	liftAnnex $ checkAssociatedDirectory curr mremote
 	((result, form), enctype) <- liftH $
-		runFormPostNoToken $ renderBootstrap $ editRepositoryAForm (isNothing mremote) curr
+		runFormPostNoToken $ renderBootstrap3 bootstrapFormLayout $ editRepositoryAForm mremote curr
 	case result of
 		FormSuccess input -> liftH $ do
 			setRepoConfig uuid mremote curr input
@@ -264,6 +276,27 @@ getUpgradeRepositoryR r = go =<< liftAnnex (repoIdRemote r)
 		liftAnnex $ setConfig 
 			(remoteConfig (Remote.repo rmt) "ignore")
 			(Git.Config.boolConfig False)
-		liftAssistant $ syncRemote rmt
 		liftAnnex $ void Remote.remoteListRefresh
+		liftAssistant updateSyncRemotes
+		liftAssistant $ syncRemote rmt
 		redirect DashboardR
+
+{- If there is no currently connected remote, display an alert suggesting
+ - to set up one. -}
+connectionNeeded :: Handler ()
+connectionNeeded = whenM noconnection $ do
+	urlrender <- getUrlRender
+	void $ liftAssistant $ do
+		close <- asIO1 removeAlert
+		addAlert $ connectionNeededAlert $ AlertButton
+			{ buttonLabel = "Connect"
+			, buttonUrl = urlrender ConnectionNeededR
+			, buttonAction = Just close
+			, buttonPrimary = True
+			}
+  where
+	noconnection = S.null . currentlyConnectedRemotes <$> liftAssistant getDaemonStatus
+
+getConnectionNeededR :: Handler Html
+getConnectionNeededR = page "Connection needed" (Just Configuration) $ do
+	$(widgetFile "configurators/needconnection")
