@@ -1,6 +1,6 @@
 {- git-annex assistant webapp configurator for ssh-based remotes
  -
- - Copyright 2012-2013 Joey Hess <joey@kitenet.net>
+ - Copyright 2012-2014 Joey Hess <joey@kitenet.net>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
@@ -13,6 +13,7 @@ module Assistant.WebApp.Configurators.Ssh where
 import Assistant.WebApp.Common
 import Assistant.WebApp.Gpg
 import Assistant.Ssh
+import Annex.Ssh
 import Assistant.WebApp.MakeRemote
 import Logs.Remote
 import Remote
@@ -24,6 +25,18 @@ import Git.Types (RemoteName)
 import qualified Remote.GCrypt as GCrypt
 import Annex.UUID
 import Logs.UUID
+import Assistant.RemoteControl
+import Types.Creds
+import Assistant.CredPairCache
+import Config.Files
+import Utility.Tmp
+import Utility.FileMode
+import Utility.ThreadScheduler
+import Utility.Env
+
+#ifdef mingw32_HOST_OS
+import Utility.Rsync
+#endif
 
 import qualified Data.Text as T
 import qualified Data.Map as M
@@ -36,10 +49,21 @@ sshConfigurator = page "Add a remote server" (Just Configuration)
 data SshInput = SshInput
 	{ inputHostname :: Maybe Text
 	, inputUsername :: Maybe Text
+	, inputAuthMethod :: AuthMethod
+	, inputPassword :: Maybe Text
 	, inputDirectory :: Maybe Text
 	, inputPort :: Int
 	}
-	deriving (Show)
+
+data AuthMethod
+	= Password
+	| CachedPassword
+	| ExistingSshKey
+	deriving (Eq, Show)
+
+-- Is a repository a new one that's being created, or did it already exist
+-- and is just being added.
+data RepoStatus = NewRepo | ExistingRepo
 
 {- SshInput is only used for applicative form prompting, this converts
  - the result of such a form into a SshData. -}
@@ -60,6 +84,8 @@ mkSshInput :: SshData -> SshInput
 mkSshInput s = SshInput
 	{ inputHostname = Just $ sshHostName s
 	, inputUsername = sshUserName s
+	, inputAuthMethod = if needsPubKey s then CachedPassword else ExistingSshKey
+	, inputPassword = Nothing
 	, inputDirectory = Just $ sshDirectory s
 	, inputPort = sshPort s
 	}
@@ -69,12 +95,22 @@ sshInputAForm :: Field Handler Text -> SshInput -> AForm Handler SshInput
 #else
 sshInputAForm :: Field WebApp WebApp Text -> SshInput -> AForm WebApp WebApp SshInput
 #endif
-sshInputAForm hostnamefield def = SshInput
-	<$> aopt check_hostname "Host name" (Just $ inputHostname def)
-	<*> aopt check_username "User name" (Just $ inputUsername def)
-	<*> aopt textField "Directory" (Just $ Just $ fromMaybe (T.pack gitAnnexAssistantDefaultDir) $ inputDirectory def)
-	<*> areq intField "Port" (Just $ inputPort def)
+sshInputAForm hostnamefield def = normalize <$> gen
   where
+	gen = SshInput
+		<$> aopt check_hostname (bfs "Host name") (Just $ inputHostname def)
+		<*> aopt check_username (bfs "User name") (Just $ inputUsername def)
+		<*> areq (selectFieldList authmethods) (bfs "Authenticate with") (Just $ inputAuthMethod def)
+		<*> aopt passwordField (bfs "Password") Nothing
+		<*> aopt textField (bfs "Directory") (Just $ Just $ fromMaybe (T.pack gitAnnexAssistantDefaultDir) $ inputDirectory def)
+		<*> areq intField (bfs "Port") (Just $ inputPort def)
+	
+	authmethods :: [(Text, AuthMethod)]
+	authmethods =
+		[ ("password", Password)
+		, ("existing ssh key", ExistingSshKey)
+		]
+
 	check_username = checkBool (all (`notElem` "/:@ \t") . T.unpack)
 		bad_username textField
 
@@ -99,6 +135,13 @@ sshInputAForm hostnamefield def = SshInput
 	check_hostname = hostnamefield -- unchecked
 #endif
 
+	-- The directory is implicitly in home, so remove any leading ~/
+	normalize i = i { inputDirectory = normalizedir <$> inputDirectory i }
+	normalizedir d
+		| "~/" `T.isPrefixOf` d = T.drop 2 d
+		| "/~/" `T.isPrefixOf` d = T.drop 3 d
+		| otherwise = d
+
 data ServerStatus
 	= UntestedServer
 	| UnusableServer Text -- reason why it's not usable
@@ -115,11 +158,11 @@ postAddSshR :: Handler Html
 postAddSshR = sshConfigurator $ do
 	username <- liftIO $ T.pack <$> myUserName
 	((result, form), enctype) <- liftH $
-		runFormPostNoToken $ renderBootstrap $ sshInputAForm textField $
-			SshInput Nothing (Just username) Nothing 22
+		runFormPostNoToken $ renderBootstrap3 bootstrapFormLayout $ sshInputAForm textField $
+			SshInput Nothing (Just username) Password Nothing Nothing 22
 	case result of
 		FormSuccess sshinput -> do
-			s <- liftIO $ testServer sshinput
+			s <- liftAssistant $ testServer sshinput
 			case s of
 				Left status -> showform form enctype status
 				Right (sshdata, u) -> liftH $ redirect $ ConfirmSshR sshdata u
@@ -136,7 +179,7 @@ sshSetupModal sshdata = $(widgetFile "configurators/ssh/setupmodal")
 getEnableRsyncR :: UUID -> Handler Html
 getEnableRsyncR = postEnableRsyncR
 postEnableRsyncR :: UUID -> Handler Html
-postEnableRsyncR = enableSpecialSshRemote getsshinput enableRsyncNet enablersync
+postEnableRsyncR = enableSshRemote getsshinput enableRsyncNet enablersync
   where
 	enablersync sshdata u = redirect $ ConfirmSshR
 		(sshdata { sshCapabilities = [RsyncCapable] }) u
@@ -148,32 +191,40 @@ getEnableSshGCryptR :: UUID -> Handler Html
 getEnableSshGCryptR = postEnableSshGCryptR
 postEnableSshGCryptR :: UUID -> Handler Html
 postEnableSshGCryptR u = whenGcryptInstalled $
-	enableSpecialSshRemote getsshinput enableRsyncNetGCrypt enablegcrypt u
+	enableSshRemote getsshinput enableRsyncNetGCrypt enablegcrypt u
   where
-  	enablegcrypt sshdata _ = prepSsh True sshdata $ \sshdata' ->
+  	enablegcrypt sshdata _ = prepSsh False sshdata $ \sshdata' ->
 		sshConfigurator $
 			checkExistingGCrypt sshdata' $
 				error "Expected to find an encrypted git repository, but did not."
 	getsshinput = parseSshUrl <=< M.lookup "gitrepo"
 
-{- To enable a special remote that uses ssh as its transport, 
- - parse a config key to get its url, and display a form whose
- - only real purpose is to check if ssh public keys need to be
- - set up.
+getEnableSshGitRemoteR :: UUID -> Handler Html
+getEnableSshGitRemoteR = postEnableSshGitRemoteR
+postEnableSshGitRemoteR :: UUID -> Handler Html
+postEnableSshGitRemoteR = enableSshRemote getsshinput enableRsyncNet enablesshgitremote
+  where
+	enablesshgitremote sshdata u = redirect $ ConfirmSshR sshdata u
+
+	getsshinput = parseSshUrl <=< M.lookup "location"
+
+{- To enable a remote that uses ssh as its transport, 
+ - parse a config key to get its url, and display a form
+ - to prompt for its password.
  -}
-enableSpecialSshRemote :: (RemoteConfig -> Maybe SshData) -> (SshInput -> RemoteName -> Handler Html) -> (SshData -> UUID -> Handler Html) -> UUID -> Handler Html
-enableSpecialSshRemote getsshinput rsyncnetsetup genericsetup u = do
+enableSshRemote :: (RemoteConfig -> Maybe SshData) -> (SshInput -> RemoteName -> Handler Html) -> (SshData -> UUID -> Handler Html) -> UUID -> Handler Html
+enableSshRemote getsshinput rsyncnetsetup genericsetup u = do
 	m <- fromMaybe M.empty . M.lookup u <$> liftAnnex readRemoteLog
 	case (mkSshInput . unmangle <$> getsshinput m, M.lookup "name" m) of
 		(Just sshinput, Just reponame) -> sshConfigurator $ do
 			((result, form), enctype) <- liftH $
-				runFormPostNoToken $ renderBootstrap $ sshInputAForm textField sshinput
+				runFormPostNoToken $ renderBootstrap3 bootstrapFormLayout $ sshInputAForm textField sshinput
 			case result of
 				FormSuccess sshinput'
 					| isRsyncNet (inputHostname sshinput') ->
 						void $ liftH $ rsyncnetsetup sshinput' reponame
 					| otherwise -> do
-						s <- liftIO $ testServer sshinput'
+						s <- liftAssistant $ testServer sshinput'
 						case s of
 							Left status -> showform form enctype status
 							Right (sshdata, _u) -> void $ liftH $ genericsetup
@@ -189,55 +240,60 @@ enableSpecialSshRemote getsshinput rsyncnetsetup genericsetup u = do
 		description <- liftAnnex $ T.pack <$> prettyUUID u
 		$(widgetFile "configurators/ssh/enable")
 
-{- Test if we can ssh into the server.
- -
- - Two probe attempts are made. First, try sshing in using the existing
- - configuration, but don't let ssh prompt for any password. If
- - passwordless login is already enabled, use it. Otherwise,
- - a special ssh key will need to be generated just for this server.
+{- To deal with git-annex and possibly even git and rsync not being
+ - available in the remote server's PATH, when git-annex was installed
+ - from the standalone tarball etc, look for a ~/.ssh/git-annex-wrapper
+ - and if it's there, use it to run a command. -}
+wrapCommand :: String -> String
+wrapCommand cmd = "if [ -x " ++ commandWrapper ++ " ]; then " ++ commandWrapper ++ " " ++ cmd ++ "; else " ++ cmd ++ "; fi"
+
+commandWrapper :: String
+commandWrapper = "~/.ssh/git-annex-wrapper"
+
+{- Test if we can ssh into the server, using the specified AuthMethod.
  -
  - Once logged into the server, probe to see if git-annex-shell,
  - git, and rsync are available. 
- - Note that, ~/.ssh/git-annex-shell may be
- - present, while git-annex-shell is not in PATH.
+ -
+ - Note that ~/.ssh/git-annex-shell may be present, while
+ - git-annex-shell is not in PATH.
+ - Also, git and rsync may not be in PATH; as long as the commandWrapper
+ - is present, assume it is able to be used to run them.
  -
  - Also probe to see if there is already a git repository at the location
  - with either an annex-uuid or a gcrypt-id set. (If not, returns NoUUID.)
  -}
-testServer :: SshInput -> IO (Either ServerStatus (SshData, UUID))
+testServer :: SshInput -> Assistant (Either ServerStatus (SshData, UUID))
 testServer (SshInput { inputHostname = Nothing }) = return $
 	Left $ UnusableServer "Please enter a host name."
 testServer sshinput@(SshInput { inputHostname = Just hn }) = do
-	(status, u) <- probe [sshOpt "NumberOfPasswordPrompts" "0"]
+	(status, u) <- probe
 	case capabilities status of
-		[] -> do
-			(status', u') <- probe []
-			case capabilities status' of
-				[] -> return $ Left status'
-				cs -> ret cs True u'
-		cs -> ret cs False u
+		[] -> return $ Left status
+		cs -> do
+			let sshdata = (mkSshData sshinput)
+				{ needsPubKey = inputAuthMethod sshinput /= ExistingSshKey
+				, sshCapabilities = cs
+				}
+			return $ Right (sshdata, u)
   where
-	ret cs needspubkey u = do
-		let sshdata = (mkSshData sshinput)
-			{ needsPubKey = needspubkey
-			, sshCapabilities = cs
-			}
-		return $ Right (sshdata, u)
-	probe extraopts = do
+	probe = do
 		let remotecommand = shellWrap $ intercalate ";"
 			[ report "loggedin"
 			, checkcommand "git-annex-shell"
 			, checkcommand "git"
 			, checkcommand "rsync"
 			, checkcommand shim
+			, checkcommand commandWrapper
 			, getgitconfig (T.unpack <$> inputDirectory sshinput)
 			]
-		knownhost <- knownHost hn
-		let sshopts = filter (not . null) $ extraopts ++
+		knownhost <- liftIO $ knownHost hn
+		let sshopts =
 			{- If this is an already known host, let
 			 - ssh check it as usual.
 			 - Otherwise, trust the host key. -}
-			[ if knownhost then "" else sshOpt "StrictHostKeyChecking" "no"
+			[ sshOpt "StrictHostKeyChecking" $
+				if knownhost then "yes" else "no"
 			, "-n" -- don't read from stdin
 			, "-p", show (inputPort sshinput)
 			, genSshHost
@@ -245,13 +301,15 @@ testServer sshinput@(SshInput { inputHostname = Just hn }) = do
 				(inputUsername sshinput)
 			, remotecommand
 			]
-		parsetranscript . fst <$> sshTranscript sshopts Nothing
+		parsetranscript . fst <$> sshAuthTranscript sshinput sshopts Nothing
 	parsetranscript s =
 		let cs = map snd $ filter (reported . fst)
 			[ ("git-annex-shell", GitAnnexShellCapable)
 			, (shim, GitAnnexShellCapable)
 			, ("git", GitCapable)
 			, ("rsync", RsyncCapable)
+			, (commandWrapper, GitCapable)
+			, (commandWrapper, RsyncCapable)
 			]
 		    u = fromMaybe NoUUID $ headMaybe $ mapMaybe finduuid $
 			map (separate (== '=')) $ lines s
@@ -270,26 +328,92 @@ testServer sshinput@(SshInput { inputHostname = Just hn }) = do
 	
 	checkcommand c = "if which " ++ c ++ "; then " ++ report c ++ "; fi"
 	token r = "git-annex-probe " ++ r
-	report r = "echo " ++ token r
+	report r = "echo " ++ shellEscape (token r)
 	shim = "~/.ssh/git-annex-shell"
 	getgitconfig (Just d)
 		| not (null d) = "cd " ++ shellEscape d ++ " && git config --list"
 	getgitconfig _ = "echo"
 
-{- Runs a ssh command; if it fails shows the user the transcript,
- - and if it succeeds, runs an action. -}
-sshSetup :: [String] -> String -> Handler Html -> Handler Html
-sshSetup opts input a = do
-	(transcript, ok) <- liftIO $ sshTranscript opts (Just input)
+{- Runs a ssh command to set up the repository; if it fails shows
+ - the user the transcript, and if it succeeds, runs an action. -}
+sshSetup :: SshInput -> [String] -> Maybe String -> Handler Html -> Handler Html
+sshSetup sshinput opts input a = do
+	(transcript, ok) <- liftAssistant $ sshAuthTranscript sshinput opts input
 	if ok
-		then a
-		else showSshErr transcript
+		then do
+			liftAssistant $ expireCachedCred $ getLogin sshinput
+			a
+		else sshErr sshinput transcript
 
-showSshErr :: String -> Handler Html
-showSshErr msg = sshConfigurator $
-	$(widgetFile "configurators/ssh/error")
+sshErr :: SshInput -> String -> Handler Html
+sshErr sshinput msg
+	| inputAuthMethod sshinput == CachedPassword =
+		ifM (liftAssistant $ isNothing <$> getCachedCred (getLogin sshinput))
+			( sshConfigurator $
+				$(widgetFile "configurators/ssh/expiredpassword")
+			, showerr
+			)
+	| otherwise = showerr
+  where
+	showerr = sshConfigurator $
+		$(widgetFile "configurators/ssh/error")
 
-{- The UUID will be NoUUID when the repository does not already exist. -}
+{- Runs a ssh command, returning a transcript of its output.
+ -
+ - Depending on the SshInput, avoids using a password, or uses a
+ - cached password. ssh is coaxed to use git-annex as SSH_ASKPASS
+ - to get the password.
+ -
+ - Note that ssh will only use SSH_ASKPASS when DISPLAY is set and there
+ - is no controlling terminal. On Unix, that is set up when the assistant
+ - starts, by calling createSession. On Windows, all of stdin, stdout, and
+ - stderr must be disconnected from the terminal. This is accomplished
+ - by always providing input on stdin.
+ -}
+sshAuthTranscript :: SshInput -> [String] -> (Maybe String) -> Assistant (String, Bool)
+sshAuthTranscript sshinput opts input = case inputAuthMethod sshinput of
+	ExistingSshKey -> liftIO $ go [passwordprompts 0] Nothing
+	CachedPassword -> setupAskPass
+	Password -> do
+		cacheCred (login, geti inputPassword) (Seconds $ 60 * 10)
+		setupAskPass
+  where
+	login = getLogin sshinput
+	geti f = maybe "" T.unpack (f sshinput)
+
+	go extraopts environ = processTranscript' "ssh" (extraopts ++ opts) environ $
+		Just (fromMaybe "" input)
+
+	setupAskPass = do
+		program <- liftIO readProgramFile
+		v <- getCachedCred login
+		liftIO $ case v of
+			Nothing -> go [passwordprompts 0] Nothing
+			Just pass -> withTmpFile "ssh" $ \passfile h -> do
+				hClose h
+				writeFileProtected passfile pass
+				environ <- getEnvironment
+				let environ' = addEntries
+					[ ("SSH_ASKPASS", program)
+					, (sshAskPassEnv, passfile)
+					-- ssh does not use SSH_ASKPASS
+					-- unless DISPLAY is set, and
+					-- there is no controlling
+					-- terminal.
+					, ("DISPLAY", ":0")
+					] environ
+				go [passwordprompts 1] (Just environ')
+	
+	passwordprompts :: Int -> String
+	passwordprompts = sshOpt "NumberOfPasswordPrompts" . show
+
+getLogin :: SshInput -> Login
+getLogin sshinput = geti inputUsername ++ "@" ++ geti inputHostname
+  where
+	geti f = maybe "" T.unpack (f sshinput)
+
+{- The UUID will be NoUUID when the repository does not already exist,
+ - or was not a git-annex repository before. -}
 getConfirmSshR :: SshData -> UUID -> Handler Html
 getConfirmSshR sshdata u
 	| u == NoUUID = handlenew
@@ -306,9 +430,7 @@ getConfirmSshR sshdata u
 		m <- liftAnnex readRemoteLog
 		case M.lookup "type" =<< M.lookup u m of
 			Just "gcrypt" -> combineExistingGCrypt sshdata' u
-			-- This handles enabling git repositories
-			-- that already exist.
-			_ -> makeSshRepo sshdata'
+			_ -> makeSshRepo ExistingRepo sshdata'
 
 {- The user has confirmed they want to combine with a ssh repository,
  - which is not known to us. So it might be using gcrypt. -}
@@ -316,18 +438,19 @@ getCombineSshR :: SshData -> Handler Html
 getCombineSshR sshdata = prepSsh False sshdata $ \sshdata' ->
 	sshConfigurator $
 		checkExistingGCrypt sshdata' $
-			void $ liftH $ makeSshRepo sshdata'
+			void $ liftH $ makeSshRepo ExistingRepo sshdata'
 
 getRetrySshR :: SshData -> Handler ()
 getRetrySshR sshdata = do
-	s <- liftIO $ testServer $ mkSshInput sshdata
+	s <- liftAssistant $ testServer $ mkSshInput sshdata
 	redirect $ either (const $ ConfirmSshR sshdata NoUUID) (uncurry ConfirmSshR) s
 
+{- Making a new git repository. -}
 getMakeSshGitR :: SshData -> Handler Html
-getMakeSshGitR sshdata = prepSsh False sshdata makeSshRepo
+getMakeSshGitR sshdata = prepSsh True sshdata (makeSshRepo NewRepo)
 
 getMakeSshRsyncR :: SshData -> Handler Html
-getMakeSshRsyncR sshdata = prepSsh False (rsyncOnly sshdata) makeSshRepo
+getMakeSshRsyncR sshdata = prepSsh False (rsyncOnly sshdata) (makeSshRepo NewRepo)
 
 rsyncOnly :: SshData -> SshData
 rsyncOnly sshdata = sshdata { sshCapabilities = [RsyncCapable] }
@@ -336,7 +459,7 @@ getMakeSshGCryptR :: SshData -> RepoKey -> Handler Html
 getMakeSshGCryptR sshdata NoRepoKey = whenGcryptInstalled $
 	withNewSecretKey $ getMakeSshGCryptR sshdata . RepoKey
 getMakeSshGCryptR sshdata (RepoKey keyid) = whenGcryptInstalled $
-	prepSsh True sshdata $ makeGCryptRepo keyid
+	prepSsh False sshdata $ makeGCryptRepo NewRepo keyid
 	
 {- Detect if the user entered a location with an existing, known
  - gcrypt repository, and enable it. Otherwise, runs the action. -}
@@ -352,10 +475,11 @@ checkExistingGCrypt sshdata nope = checkGCryptRepoEncryption repourl nope nope $
 
 {- Enables an existing gcrypt special remote. -}
 enableGCrypt :: SshData -> RemoteName -> Handler Html
-enableGCrypt sshdata reponame = 
-	setupCloudRemote TransferGroup Nothing $ 
-		enableSpecialRemote reponame GCrypt.remote $ M.fromList
-			[("gitrepo", genSshUrl sshdata)]
+enableGCrypt sshdata reponame = setupRemote postsetup Nothing Nothing mk
+  where
+	mk = enableSpecialRemote reponame GCrypt.remote Nothing $
+		M.fromList [("gitrepo", genSshUrl sshdata)]
+	postsetup _ = redirect DashboardR
 
 {- Combining with a gcrypt repository that may not be
  - known in remote.log, so probe the gcrypt repo. -}
@@ -368,50 +492,82 @@ combineExistingGCrypt sshdata u = do
 
 {- Sets up remote repository for ssh, or directory for rsync. -}
 prepSsh :: Bool -> SshData -> (SshData -> Handler Html) -> Handler Html
-prepSsh newgcrypt sshdata a
+prepSsh needsinit sshdata a
 	| needsPubKey sshdata = do
 		keypair <- liftIO genSshKeyPair
 		sshdata' <- liftIO $ setupSshKeyPair keypair sshdata
-		prepSsh' newgcrypt sshdata sshdata' (Just keypair) a
+		prepSsh' needsinit sshdata sshdata' (Just keypair) a
 	| sshPort sshdata /= 22 = do
 		sshdata' <- liftIO $ setSshConfig sshdata []
-		prepSsh' newgcrypt sshdata sshdata' Nothing a
-	| otherwise = prepSsh' newgcrypt sshdata sshdata Nothing a
+		prepSsh' needsinit sshdata sshdata' Nothing a
+	| otherwise = prepSsh' needsinit sshdata sshdata Nothing a
 
 prepSsh' :: Bool -> SshData -> SshData -> Maybe SshKeyPair -> (SshData -> Handler Html) -> Handler Html
-prepSsh' newgcrypt origsshdata sshdata keypair a = sshSetup
+prepSsh' needsinit origsshdata sshdata keypair a = sshSetup (mkSshInput origsshdata)
 	 [ "-p", show (sshPort origsshdata)
 	 , genSshHost (sshHostName origsshdata) (sshUserName origsshdata)
 	 , remoteCommand
-	 ] "" (a sshdata)
+	 ] Nothing (a sshdata)
   where
 	remotedir = T.unpack $ sshDirectory sshdata
 	remoteCommand = shellWrap $ intercalate "&&" $ catMaybes
 		[ Just $ "mkdir -p " ++ shellEscape remotedir
 		, Just $ "cd " ++ shellEscape remotedir
-		, if rsynconly then Nothing else Just "if [ ! -d .git ]; then git init --bare --shared && git config receive.denyNonFastforwards false; fi"
-		, if rsynconly || newgcrypt then Nothing else Just "git annex init"
+		, if rsynconly then Nothing else Just $ unwords
+			[ "if [ ! -d .git ]; then"
+			, wrapCommand "git init --bare --shared"
+		, "&&"
+			, wrapCommand "git config receive.denyNonFastforwards"
+			, ";fi"
+			]
+		, if needsinit then Just (wrapCommand "git annex init") else Nothing
 		, if needsPubKey origsshdata
 			then addAuthorizedKeysCommand (hasCapability origsshdata GitAnnexShellCapable) remotedir . sshPubKey <$> keypair
 			else Nothing
 		]
 	rsynconly = onlyCapability origsshdata RsyncCapable
 
-makeSshRepo :: SshData -> Handler Html
-makeSshRepo sshdata = setupCloudRemote TransferGroup Nothing $
-	makeSshRemote sshdata
+makeSshRepo :: RepoStatus -> SshData -> Handler Html
+makeSshRepo rs sshdata
+	| onlyCapability sshdata RsyncCapable = setupCloudRemote TransferGroup Nothing mk
+	| otherwise = makeSshRepoConnection rs mk setup
+  where
+	mk = makeSshRemote sshdata
+	-- Record the location of the ssh remote in the remote log, so it
+	-- can easily be enabled elsewhere using the webapp.
+	setup r = do
+		m <- readRemoteLog
+		let c = fromMaybe M.empty (M.lookup (Remote.uuid r) m)
+		let c' = M.insert "location" (genSshUrl sshdata) $
+			M.insert "type" "git" $
+			M.insert "name" (fromMaybe (Remote.name r) (M.lookup "name" c)) c
+		configSet (Remote.uuid r) c'
 
-makeGCryptRepo :: KeyId -> SshData -> Handler Html
-makeGCryptRepo keyid sshdata = setupCloudRemote TransferGroup Nothing $ 
-	makeGCryptRemote (sshRepoName sshdata) (genSshUrl sshdata) keyid
+makeSshRepoConnection :: RepoStatus -> Annex RemoteName -> (Remote -> Annex ()) -> Handler Html
+makeSshRepoConnection rs mk setup = setupRemote postsetup mgroup Nothing mk
+  where
+	mgroup = case rs of
+		NewRepo -> Just TransferGroup
+		ExistingRepo -> Nothing
+	postsetup r = do
+		liftAssistant $ sendRemoteControl RELOAD
+		liftAnnex $ setup r
+		case rs of
+			NewRepo -> redirect $ EditNewRepositoryR (Remote.uuid r)
+			ExistingRepo -> redirect DashboardR
+
+makeGCryptRepo :: RepoStatus -> KeyId -> SshData -> Handler Html
+makeGCryptRepo rs keyid sshdata = makeSshRepoConnection rs mk (const noop)
+  where
+	mk = makeGCryptRemote (sshRepoName sshdata) (genSshUrl sshdata) keyid
 
 getAddRsyncNetR :: Handler Html
 getAddRsyncNetR = postAddRsyncNetR
 postAddRsyncNetR :: Handler Html
 postAddRsyncNetR = do
 	((result, form), enctype) <- runFormPostNoToken $
-		renderBootstrap $ sshInputAForm hostnamefield $
-			SshInput Nothing Nothing Nothing 22
+		renderBootstrap3 bootstrapFormLayout $ sshInputAForm hostnamefield $
+			SshInput Nothing Nothing Password Nothing Nothing 22
 	let showform status = inpage $
 		$(widgetFile "configurators/rsync.net/add")
 	case result of
@@ -436,6 +592,7 @@ postAddRsyncNetR = do
 	go sshinput = do
 		let reponame = genSshRepoName "rsync.net" 
 			(maybe "" T.unpack $ inputDirectory sshinput)
+		
 		prepRsyncNet sshinput reponame $ \sshdata -> inpage $ 
 			checkExistingGCrypt sshdata $ do
 				secretkeys <- sortBy (comparing snd) . M.toList
@@ -443,21 +600,22 @@ postAddRsyncNetR = do
 				$(widgetFile "configurators/rsync.net/encrypt")
 
 getMakeRsyncNetSharedR :: SshData -> Handler Html
-getMakeRsyncNetSharedR = makeSshRepo . rsyncOnly
+getMakeRsyncNetSharedR = makeSshRepo NewRepo . rsyncOnly
 
-{- Make a gcrypt special remote on rsync.net. -}
+{- Make a new gcrypt special remote on rsync.net. -}
 getMakeRsyncNetGCryptR :: SshData -> RepoKey -> Handler Html
 getMakeRsyncNetGCryptR sshdata NoRepoKey = whenGcryptInstalled $
 	withNewSecretKey $ getMakeRsyncNetGCryptR sshdata . RepoKey
 getMakeRsyncNetGCryptR sshdata (RepoKey keyid) = whenGcryptInstalled $
-	sshSetup [sshhost, gitinit] [] $ makeGCryptRepo keyid sshdata
+	sshSetup (mkSshInput sshdata) [sshhost, gitinit] Nothing $
+		makeGCryptRepo NewRepo keyid sshdata
   where
 	sshhost = genSshHost (sshHostName sshdata) (sshUserName sshdata)
 	gitinit = "git init --bare " ++ T.unpack (sshDirectory sshdata)
 
 enableRsyncNet :: SshInput -> String -> Handler Html
 enableRsyncNet sshinput reponame = 
-	prepRsyncNet sshinput reponame $ makeSshRepo . rsyncOnly
+	prepRsyncNet sshinput reponame $ makeSshRepo ExistingRepo . rsyncOnly
 
 enableRsyncNetGCrypt :: SshInput -> RemoteName -> Handler Html
 enableRsyncNetGCrypt sshinput reponame = 
@@ -468,8 +626,13 @@ enableRsyncNetGCrypt sshinput reponame =
 	notencrypted = error "Unexpectedly found a non-encrypted git repository, instead of the expected encrypted git repository."
 	notinstalled = error "internal"
 
-{- Prepares rsync.net ssh key, and if successful, runs an action with
- - its SshData. -}
+{- Prepares rsync.net ssh key and creates the directory that will be 
+ - used on rsync.net. If successful, runs an action with its SshData.
+ -
+ - To append the ssh key to rsync.net's authorized_keys, their
+ - documentation recommends a dd methodd, where the line is fed
+ - in to ssh over stdin.
+ -}
 prepRsyncNet :: SshInput -> String -> (SshData -> Handler Html) -> Handler Html
 prepRsyncNet sshinput reponame a = do
 	knownhost <- liftIO $ maybe (return False) knownHost (inputHostname sshinput)
@@ -480,25 +643,21 @@ prepRsyncNet sshinput reponame a = do
 			, needsPubKey = True
 			, sshCapabilities = [RsyncCapable]
 			}
+	let sshhost = genSshHost (sshHostName sshdata) (sshUserName sshdata)
+	let torsyncnet cmd = filter (not . null)
+		[ if knownhost then "" else sshOpt "StrictHostKeyChecking" "no"
+		, sshhost
+		, cmd
+		]
 	{- I'd prefer to separate commands with && , but
-	 - rsync.net's shell does not support that.
-	 -
-	 - The dd method of appending to the authorized_keys file is the
-	 - one recommended by rsync.net documentation. I touch the file first
-	 - to not need to use a different method to create it.
-	 -}
+	 - rsync.net's shell does not support that. -}
 	let remotecommand = intercalate ";"
 		[ "mkdir -p .ssh"
 		, "touch .ssh/authorized_keys"
 		, "dd of=.ssh/authorized_keys oflag=append conv=notrunc"
 		, "mkdir -p " ++ T.unpack (sshDirectory sshdata)
 		]
-	let sshopts = filter (not . null)
-		[ if knownhost then "" else sshOpt "StrictHostKeyChecking" "no"
-		, genSshHost (sshHostName sshdata) (sshUserName sshdata)
-		, remotecommand
-		]
-	sshSetup sshopts (sshPubKey keypair) $ a sshdata
+	sshSetup sshinput (torsyncnet remotecommand) (Just $ sshPubKey keypair) (a sshdata)
 
 isRsyncNet :: Maybe Text -> Bool
 isRsyncNet Nothing = False
