@@ -31,10 +31,12 @@ import qualified Data.Map as M
 import qualified Data.Set as S
 import qualified Data.Text as T
 import Data.Function
+import Control.Concurrent
 
-type RepoList = [(RepoDesc, RepoId, Actions)]
+type RepoList = [(RepoDesc, RepoId, CurrentlyConnected, Actions)]
 
 type RepoDesc = String
+type CurrentlyConnected = Bool
 
 {- Actions that can be performed on a repo in the list. -}
 data Actions
@@ -111,10 +113,10 @@ cloudRepoList = repoListDisplay RepoSelector
 repoListDisplay :: RepoSelector -> Widget
 repoListDisplay reposelector = do
 	autoUpdate ident (NotifierRepoListR reposelector) (10 :: Int) (10 :: Int)
-	addScript $ StaticR jquery_ui_core_js
-	addScript $ StaticR jquery_ui_widget_js
-	addScript $ StaticR jquery_ui_mouse_js
-	addScript $ StaticR jquery_ui_sortable_js
+	addScript $ StaticR js_jquery_ui_core_js
+	addScript $ StaticR js_jquery_ui_widget_js
+	addScript $ StaticR js_jquery_ui_mouse_js
+	addScript $ StaticR js_jquery_ui_sortable_js
 
 	repolist <- liftH $ repoList reposelector
 	let addmore = nudgeAddMore reposelector
@@ -164,7 +166,7 @@ repoList reposelector
 		| Remote.readonly r = False
 		| onlyCloud reposelector = Git.repoIsUrl (Remote.repo r)
 			&& Remote.uuid r /= NoUUID 
-			&& not (isXMPPRemote r)
+			&& not (Remote.isXMPPRemote r)
 		| otherwise = True
 	selectedremote Nothing = False
 	selectedremote (Just (iscloud, _))
@@ -187,17 +189,28 @@ repoList reposelector
 				Just rr	| remoteLocationIsUrl (parseRemoteLocation rr g) ->
 					val True EnableSshGCryptR
 				_ -> Nothing
+		Just "git" -> 
+			case getconfig "location" of
+				Just loc | remoteLocationIsSshUrl (parseRemoteLocation loc g) ->
+					val True EnableSshGitRemoteR
+				_ -> Nothing
 		_ -> Nothing
 	  where
 	  	getconfig k = M.lookup k =<< M.lookup u m
 		val iscloud r = Just (iscloud, (RepoUUID u, DisabledRepoActions $ r u))
-	list l = liftAnnex $
+	list l = do
+		cc <- currentlyConnectedRemotes <$> liftAssistant getDaemonStatus
 		forM (nubBy ((==) `on` fst) l) $ \(repoid, actions) ->
-			(,,)
-				<$> describeRepoId repoid
+			(,,,)
+				<$> liftAnnex (describeRepoId repoid)
 				<*> pure repoid
+				<*> pure (getCurrentlyConnected repoid cc)
 				<*> pure actions
 
+getCurrentlyConnected :: RepoId -> S.Set UUID -> CurrentlyConnected
+getCurrentlyConnected (RepoUUID u) cc = S.member u cc
+getCurrentlyConnected _ _ = False
+
 getEnableSyncR :: RepoId -> Handler ()
 getEnableSyncR = flipSync True
 
@@ -215,17 +228,17 @@ getRepositoriesReorderR = do
 	{- Get uuid of the moved item, and the list it was moved within. -}
 	moved <- fromjs <$> runInputGet (ireq textField "moved")
 	list <- map fromjs <$> lookupGetParams "list[]"
-	liftAnnex $ go list =<< Remote.remoteFromUUID moved
+	liftAnnex $ go list =<< repoIdRemote moved
 	liftAssistant updateSyncRemotes
   where
 	go _ Nothing = noop
   	go list (Just remote) = do
-		rs <- catMaybes <$> mapM Remote.remoteFromUUID list
+		rs <- catMaybes <$> mapM repoIdRemote list
 		forM_ (reorderCosts remote rs) $ \(r, newcost) ->
 			when (Remote.cost r /= newcost) $
 				setRemoteCost (Remote.repo r) newcost
 		void remoteListRefresh
-  	fromjs = toUUID . T.unpack
+  	fromjs = fromMaybe (RepoUUID NoUUID) . readish . T.unpack
 
 reorderCosts :: Remote -> [Remote] -> [(Remote, Cost)]
 reorderCosts remote rs = zip rs'' (insertCostAfter costs i)
@@ -238,3 +251,15 @@ reorderCosts remote rs = zip rs'' (insertCostAfter costs i)
 	costs = map Remote.cost rs'
 	rs'' = (\(x, y) -> x ++ [remote] ++ y) $ splitAt (i + 1) rs'
 
+getSyncNowRepositoryR :: UUID -> Handler ()
+getSyncNowRepositoryR uuid = do
+	u <- liftAnnex getUUID
+	if u == uuid
+		then do
+			thread <- liftAssistant $ asIO $
+				reconnectRemotes True
+					=<< (syncRemotes <$> getDaemonStatus)
+			void $ liftIO $ forkIO thread
+		else maybe noop (liftAssistant . syncRemote)
+			=<< liftAnnex (Remote.remoteFromUUID uuid)
+	redirectBack
