@@ -1,6 +1,6 @@
 {- git-annex key/value backends
  -
- - Copyright 2010,2013 Joey Hess <joey@kitenet.net>
+ - Copyright 2010-2014 Joey Hess <joey@kitenet.net>
  -
  - Licensed under the GNU GPL version 3 or higher.
  -}
@@ -10,6 +10,7 @@ module Backend (
 	orderedList,
 	genKey,
 	lookupFile,
+	getBackend,
 	isAnnexLink,
 	chooseBackend,
 	lookupBackendName,
@@ -27,12 +28,12 @@ import qualified Types.Backend as B
 import Config
 
 -- When adding a new backend, import it here and add it to the list.
-import qualified Backend.SHA
+import qualified Backend.Hash
 import qualified Backend.WORM
 import qualified Backend.URL
 
 list :: [Backend]
-list = Backend.SHA.backends ++ Backend.WORM.backends ++ Backend.URL.backends
+list = Backend.Hash.backends ++ Backend.WORM.backends ++ Backend.URL.backends
 
 {- List of backends in the order to try them when storing a new key. -}
 orderedList :: Annex [Backend]
@@ -74,7 +75,7 @@ genKey' (b:bs) source = do
 		| c == '\n' = '_'
 		| otherwise = c
 
-{- Looks up the key and backend corresponding to an annexed file,
+{- Looks up the key corresponding to an annexed file,
  - by examining what the file links to.
  -
  - In direct mode, there is often no link on disk, in which case
@@ -82,7 +83,7 @@ genKey' (b:bs) source = do
  - on disk still takes precedence over what was committed to git in direct
  - mode.
  -}
-lookupFile :: FilePath -> Annex (Maybe (Key, Backend))
+lookupFile :: FilePath -> Annex (Maybe Key)
 lookupFile file = do
 	mkey <- isAnnexLink file
 	case mkey of
@@ -92,14 +93,15 @@ lookupFile file = do
 			, return Nothing
 			)
   where
-	makeret k = let bname = keyBackendName k in
-		case maybeLookupBackendName bname of
-			Just backend -> return $ Just (k, backend)
-			Nothing -> do
-				warning $
-					"skipping " ++ file ++
-					" (unknown backend " ++ bname ++ ")"
-				return Nothing
+	makeret k = return $ Just k
+
+getBackend :: FilePath -> Key -> Annex (Maybe Backend)
+getBackend file k = let bname = keyBackendName k in
+	case maybeLookupBackendName bname of
+		Just backend -> return $ Just backend
+		Nothing -> do
+			warning $ "skipping " ++ file ++ " (unknown backend " ++ bname ++ ")"
+			return Nothing
 
 {- Looks up the backend that should be used for a file.
  - That can be configured on a per-file basis in the gitattributes file. -}
