@@ -44,6 +44,7 @@ genBackend hash = Just Backend
 	, getKey = keyValue hash
 	, fsckKey = Just $ checkKeyChecksum hash
 	, canUpgradeKey = Just needsUpgrade
+	, fastMigrate = Just trivialMigrate
 	}
 
 genBackendE :: Hash -> Maybe Backend
@@ -101,6 +102,7 @@ checkKeyChecksum hash key file = do
 	case (mstat, fast) of
 		(Just stat, False) -> do
 			let filesize = fromIntegral $ fileSize stat
+			showSideAction "checksum"
 			check <$> hashFile hash file filesize
 		_ -> return True
   where
@@ -128,10 +130,17 @@ needsUpgrade :: Key -> Bool
 needsUpgrade key = "\\" `isPrefixOf` keyHash key ||
 	any (not . validExtension) (takeExtensions $ keyName key)
 
+{- Fast migration from hashE to hash backend. (Optimisation) -}
+trivialMigrate :: Key -> Backend -> Maybe Key
+trivialMigrate oldkey newbackend
+	| keyBackendName oldkey == name newbackend ++ "E" = Just $ oldkey
+		{ keyName = keyHash oldkey
+		, keyBackendName = name newbackend
+		}
+	| otherwise = Nothing
+
 hashFile :: Hash -> FilePath -> Integer -> Annex String
-hashFile hash file filesize = do
-	showAction "checksum"
-	liftIO $ go hash
+hashFile hash file filesize = liftIO $ go hash
   where
   	go (SHAHash hashsize) = case shaHasher hashsize filesize of
 		Left sha -> sha <$> L.readFile file
