@@ -10,11 +10,10 @@ module Backend.URL (
 	fromUrl
 ) where
 
-import Data.Hash.MD5
-
 import Common.Annex
 import Types.Backend
 import Types.Key
+import Backend.Utilities
 
 backends :: [Backend]
 backends = [backend]
@@ -25,20 +24,15 @@ backend = Backend
 	, getKey = const $ return Nothing
 	, fsckKey = Nothing
 	, canUpgradeKey = Nothing
+	, fastMigrate = Nothing
 	}
 
-{- When it's not too long, use the full url as the key name.
- - If the url is too long, it's truncated at half the filename length
- - limit, and the md5 of the url is prepended to ensure a unique key. -}
+{- Every unique url has a corresponding key. -}
 fromUrl :: String -> Maybe Integer -> Annex Key
 fromUrl url size = do
-	limit <- liftIO . fileNameLengthLimit =<< fromRepo gitAnnexDir
-	let truncurl = truncateFilePath (limit `div` 2) url
-	let key = if url == truncurl
-		then url
-		else truncurl ++ "-" ++ md5s (Str url)
+	n <- genKeyName url
 	return $ stubKey
-		{ keyName = key
+		{ keyName = n
 		, keyBackendName = "URL"
 		, keySize = size
-	}
+		}
