@@ -11,6 +11,7 @@ import Common.Annex
 import Types.Backend
 import Types.Key
 import Types.KeySource
+import Backend.Utilities
 
 backends :: [Backend]
 backends = [backend]
@@ -21,6 +22,7 @@ backend = Backend
 	, getKey = keyValue
 	, fsckKey = Nothing
 	, canUpgradeKey = Nothing
+	, fastMigrate = Nothing
 	}
 
 {- The key includes the file size, modification time, and the
@@ -33,9 +35,10 @@ backend = Backend
 keyValue :: KeySource -> Annex (Maybe Key)
 keyValue source = do
 	stat <- liftIO $ getFileStatus $ contentLocation source
-	return $ Just Key {
-		keyName = takeFileName $ keyFilename source,
-		keyBackendName = name backend,
-		keySize = Just $ fromIntegral $ fileSize stat,
-		keyMtime = Just $ modificationTime stat
-	}
+	n <- genKeyName $ keyFilename source
+	return $ Just Key
+		{ keyName = n
+		, keyBackendName = name backend
+		, keySize = Just $ fromIntegral $ fileSize stat
+		, keyMtime = Just $ modificationTime stat
+		}
