@@ -3,26 +3,21 @@
 module Build.Configure where
 
 import System.Directory
-import Data.List
-import System.Process
 import Control.Applicative
-import System.FilePath
 import System.Environment (getArgs)
-import Data.Maybe
 import Control.Monad.IfElse
-import Data.Char
+import Control.Monad
 
 import Build.TestConfig
 import Build.Version
 import Utility.SafeCommand
-import Utility.Monad
 import Utility.ExternalSHA
 import Utility.Env
 import qualified Git.Version
 
 tests :: [TestCase]
 tests =
-	[ TestCase "version" getVersion
+	[ TestCase "version" (Config "packageversion" . StringConfig <$> getVersion)
 	, TestCase "UPGRADE_LOCATION" getUpgradeLocation
 	, TestCase "git" $ requireCmd "git" "git --version >/dev/null"
 	, TestCase "git version" getGitVersion
@@ -34,10 +29,9 @@ tests =
 	, TestCase "curl" $ testCmd "curl" "curl --version >/dev/null"
 	, TestCase "wget" $ testCmd "wget" "wget --version >/dev/null"
 	, TestCase "bup" $ testCmd "bup" "bup --version >/dev/null"
-	, TestCase "quvi" $ testCmd "quvi" "quvi --version >/dev/null"
-	, TestCase "newquvi" $ testCmd "newquvi" "quvi info >/dev/null"
 	, TestCase "nice" $ testCmd "nice" "nice true >/dev/null"
 	, TestCase "ionice" $ testCmd "ionice" "ionice -c3 true >/dev/null"
+	, TestCase "nocache" $ testCmd "nocache" "nocache true >/dev/null"
 	, TestCase "gpg" $ maybeSelectCmd "gpg"
 		[ ("gpg", "--version >/dev/null")
 		, ("gpg2", "--version >/dev/null") ]
@@ -52,8 +46,8 @@ tests =
 	, (384, "38b060a751ac96384cd9327eb1b1e36a21fdb71114be07434c0cc7bf63f6e1da274edebfe76f65fbd51ad2f14898b95b")
 	]
 
-{- shaNsum are the program names used by coreutils. Some systems like OSX
- - sometimes install these with 'g' prefixes.
+{- shaNsum are the program names used by coreutils. Some systems
+ - install these with 'g' prefixes.
  -
  - On some systems, shaN is used instead, but on other
  - systems, it might be "hashalot", which does not produce
@@ -66,21 +60,16 @@ shaTestCases l = map make l
 		Config key . MaybeStringConfig <$> search (shacmds n)
 	  where
 		key = "sha" ++ show n
-	  	search [] = return Nothing
+		search [] = return Nothing
 		search (c:cmds) = do
 			sha <- externalSHA c n "/dev/null"
 			if sha == Right knowngood
 				then return $ Just c
 				else search cmds
 	
-	shacmds n = concatMap (\x -> [x, 'g':x, osxpath </> x]) $
+	shacmds n = concatMap (\x -> [x, 'g':x]) $
 		map (\x -> "sha" ++ show n ++ x) ["sum", ""]
 
-	{- Max OSX sometimes puts GNU tools outside PATH, so look in
-	 - the location it uses, and remember where to run them
-	 - from. -}
-	osxpath = "/opt/local/libexec/gnubin"
-
 tmpDir :: String
 tmpDir = "tmp"
 
@@ -99,8 +88,12 @@ getUpgradeLocation = do
 	return $ Config "upgradelocation" $ MaybeStringConfig e
 
 getGitVersion :: Test
-getGitVersion = Config "gitversion" . StringConfig . show
-	<$> Git.Version.installed
+getGitVersion = do
+	v <- Git.Version.installed
+	let oldestallowed = Git.Version.normalize "1.7.1.0"
+	when (v < oldestallowed) $
+		error $ "installed git version " ++ show v ++ " is too old! (Need " ++ show oldestallowed ++ " or newer)"
+	return $ Config "gitversion" $ StringConfig $ show v
 
 getSshConnectionCaching :: Test
 getSshConnectionCaching = Config "sshconnectioncaching" . BoolConfig <$>
