@@ -9,8 +9,6 @@
 
 module Command.Add where
 
-import System.PosixCompat.Files
-
 import Common.Annex
 import Annex.Exception
 import Command
@@ -21,6 +19,7 @@ import Annex.Content
 import Annex.Content.Direct
 import Annex.Perms
 import Annex.Link
+import Annex.MetaData
 import qualified Annex
 import qualified Annex.Queue
 #ifdef WITH_CLIBS
@@ -35,24 +34,29 @@ import Annex.ReplaceFile
 import Utility.Tmp
 
 def :: [Command]
-def = [notBareRepo $ command "add" paramPaths seek SectionCommon
-	"add files to annex"]
+def = [notBareRepo $ withOptions [includeDotFilesOption] $
+	command "add" paramPaths seek SectionCommon
+		"add files to annex"]
+
+includeDotFilesOption :: Option
+includeDotFilesOption = flagOption [] "include-dotfiles" "don't skip dotfiles"
 
 {- Add acts on both files not checked into git yet, and unlocked files.
  -
  - In direct mode, it acts on any files that have changed. -}
-seek :: [CommandSeek]
-seek =
-	[ go withFilesNotInGit
-	, whenNotDirect $ go withFilesUnlocked
-	, whenDirect $ go withFilesMaybeModified
-	]
-  where
-  	go a = withValue largeFilesMatcher $ \matcher ->
-		a $ \file -> ifM (checkFileMatcher matcher file <||> Annex.getState Annex.force)
-			( start file
-			, stop
-			)
+seek :: CommandSeek
+seek ps = do
+	matcher <- largeFilesMatcher
+	let go a = flip a ps $ \file -> ifM (checkFileMatcher matcher file <||> Annex.getState Annex.force)
+		( start file
+		, stop
+		)
+	skipdotfiles <- not <$> Annex.getFlag (optionName includeDotFilesOption)
+	go $ withFilesNotInGit skipdotfiles
+	ifM isDirect
+		( go withFilesMaybeModified
+		, go withFilesUnlocked
+		)
 
 {- The add subcommand annexes a file, generating a key for it using a
  - backend, and then moving it into the annex directory and setting up
@@ -69,7 +73,7 @@ start file = ifAnnexed file addpresent add
 				| otherwise -> do
 					showStart "add" file
 					next $ perform file
-	addpresent (key, _) = ifM isDirect
+	addpresent key = ifM isDirect
 		( ifM (goodContent key file) ( stop , add )
 		, fixup key
 		)
@@ -94,12 +98,15 @@ start file = ifAnnexed file addpresent add
  - Lockdown can fail if a file gets deleted, and Nothing will be returned.
  -}
 lockDown :: FilePath -> Annex (Maybe KeySource)
-lockDown file = ifM crippledFileSystem
-	( liftIO $ catchMaybeIO nohardlink
-	, do
-		tmp <- fromRepo gitAnnexTmpDir
+lockDown = either (\e -> showErr e >> return Nothing) (return . Just) <=< lockDown'
+
+lockDown' :: FilePath -> Annex (Either IOException KeySource)
+lockDown' file = ifM crippledFileSystem
+	( withTSDelta $ liftIO . tryIO . nohardlink
+	, tryAnnexIO $ do
+		tmp <- fromRepo gitAnnexTmpMiscDir
 		createAnnexDirectory tmp
-		eitherToMaybe <$> tryAnnexIO (go tmp)
+		go tmp
 	)
   where
 	{- In indirect mode, the write bit is removed from the file as part
@@ -115,22 +122,22 @@ lockDown file = ifM crippledFileSystem
   	go tmp = do
 		unlessM isDirect $
 			freezeContent file
-		liftIO $ do
+		withTSDelta $ \delta -> liftIO $ do
 			(tmpfile, h) <- openTempFile tmp $
 				relatedTemplate $ takeFileName file
 			hClose h
 			nukeFile tmpfile
-			withhardlink tmpfile `catchIO` const nohardlink
-  	nohardlink = do
-		cache <- genInodeCache file
+			withhardlink delta tmpfile `catchIO` const (nohardlink delta)
+  	nohardlink delta = do
+		cache <- genInodeCache file delta
 		return KeySource
 			{ keyFilename = file
 			, contentLocation = file
 			, inodeCache = cache
 			}
-	withhardlink tmpfile = do
+	withhardlink delta tmpfile = do
 		createLink file tmpfile
-		cache <- genInodeCache tmpfile
+		cache <- genInodeCache tmpfile delta
 		return KeySource
 			{ keyFilename = file
 			, contentLocation = tmpfile
@@ -144,29 +151,35 @@ lockDown file = ifM crippledFileSystem
  -}
 ingest :: Maybe KeySource -> Annex (Maybe Key, Maybe InodeCache)
 ingest Nothing = return (Nothing, Nothing)
-ingest (Just source) = do
+ingest (Just source) = withTSDelta $ \delta -> do
 	backend <- chooseBackend $ keyFilename source
 	k <- genKey source backend
-	cache <- liftIO $ genInodeCache $ contentLocation source
-	case (cache, inodeCache source) of
-		(_, Nothing) -> go k cache
-		(Just newc, Just c) | compareStrong c newc -> go k cache
+	ms <- liftIO $ catchMaybeIO $ getFileStatus $ contentLocation source
+	mcache <- maybe (pure Nothing) (liftIO . toInodeCache delta) ms
+	case (mcache, inodeCache source) of
+		(_, Nothing) -> go k mcache ms
+		(Just newc, Just c) | compareStrong c newc -> go k mcache ms
 		_ -> failure "changed while it was being added"
   where
-	go k cache = ifM isDirect ( godirect k cache , goindirect k cache )
+	go k mcache ms = ifM isDirect
+		( godirect k mcache ms
+		, goindirect k mcache ms
+		)
 
-	goindirect (Just (key, _)) mcache = do
+	goindirect (Just (key, _)) mcache ms = do
 		catchAnnex (moveAnnex key $ contentLocation source)
 			(undo (keyFilename source) key)
+		maybe noop (genMetaData key (keyFilename source)) ms
 		liftIO $ nukeFile $ keyFilename source
 		return $ (Just key, mcache)
-	goindirect Nothing _ = failure "failed to generate a key"
+	goindirect _ _ _ = failure "failed to generate a key"
 
-	godirect (Just (key, _)) (Just cache) = do
+	godirect (Just (key, _)) (Just cache) ms = do
 		addInodeCache key cache
+		maybe noop (genMetaData key (keyFilename source)) ms
 		finishIngestDirect key source
 		return $ (Just key, Just cache)
-	godirect _ _ = failure "failed to generate a key"
+	godirect _ _ _ = failure "failed to generate a key"
 
 	failure msg = do
 		warning $ keyFilename source ++ " " ++ msg
@@ -213,15 +226,15 @@ link file key mcache = flip catchAnnex (undo file key) $ do
 	l <- inRepo $ gitAnnexLink file key
 	replaceFile file $ makeAnnexLink l
 
-#ifdef WITH_CLIBS
-#ifndef __ANDROID__
 	-- touch symlink to have same time as the original file,
 	-- as provided in the InodeCache
 	case mcache of
+#if defined(WITH_CLIBS) && ! defined(__ANDROID__)
 		Just c -> liftIO $ touch file (TimeSpec $ inodeCacheToMtime c) False
+#else
+		Just _ -> noop
+#endif
 		Nothing -> noop
-#endif
-#endif
 
 	return l
 
@@ -250,12 +263,12 @@ addLink file key mcache = ifM (coreSymlinks <$> Annex.getGitConfig)
 
 cleanup :: FilePath -> Key -> Maybe InodeCache -> Bool -> CommandCleanup
 cleanup file key mcache hascontent = do
-	when hascontent $
-		logStatus key InfoPresent
 	ifM (isDirect <&&> pure hascontent)
 		( do
 			l <- inRepo $ gitAnnexLink file key
 			stageSymlink file =<< hashSymlink l
 		, addLink file key mcache
 		)
+	when hascontent $
+		logStatus key InfoPresent
 	return True
