@@ -21,14 +21,12 @@ import qualified Annex.Url as Url
 import qualified Backend.URL
 import Annex.Content
 import Logs.Web
-import qualified Option
 import Types.Key
 import Types.KeySource
 import Config
 import Annex.Content.Direct
 import Logs.Location
-import qualified Logs.Transfer as Transfer
-import Utility.Daemon (checkDaemon)
+import qualified Annex.Transfer as Transfer
 #ifdef WITH_QUVI
 import Annex.Quvi
 import qualified Utility.Quvi as Quvi
@@ -40,19 +38,20 @@ def = [notBareRepo $ withOptions [fileOption, pathdepthOption, relaxedOption] $
 		SectionCommon "add urls to annex"]
 
 fileOption :: Option
-fileOption = Option.field [] "file" paramFile "specify what file the url is added to"
+fileOption = fieldOption [] "file" paramFile "specify what file the url is added to"
 
 pathdepthOption :: Option
-pathdepthOption = Option.field [] "pathdepth" paramNumber "path components to use in filename"
+pathdepthOption = fieldOption [] "pathdepth" paramNumber "path components to use in filename"
 
 relaxedOption :: Option
-relaxedOption = Option.flag [] "relaxed" "skip size check"
+relaxedOption = flagOption [] "relaxed" "skip size check"
 
-seek :: [CommandSeek]
-seek = [withField fileOption return $ \f ->
-	withFlag relaxedOption $ \relaxed ->
-	withField pathdepthOption (return . maybe Nothing readish) $ \d ->
-	withStrings $ start relaxed f d]
+seek :: CommandSeek
+seek ps = do
+	f <- getOptionField fileOption return
+	relaxed <- getOptionFlag relaxedOption
+	d <- getOptionField pathdepthOption (return . maybe Nothing readish)
+	withStrings (start relaxed f d) ps
 
 start :: Bool -> Maybe FilePath -> Maybe Int -> String -> CommandStart
 start relaxed optfile pathdepth s = go $ fromMaybe bad $ parseURI s
@@ -65,7 +64,7 @@ start relaxed optfile pathdepth s = go $ fromMaybe bad $ parseURI s
 		QuviDownloader -> usequvi
 		DefaultDownloader -> 
 #ifdef WITH_QUVI
-			ifM (liftIO $ Quvi.supported s')
+			ifM (quviSupported s')
 				( usequvi
 				, regulardownload url
 				)
@@ -97,45 +96,64 @@ performQuvi :: Bool -> URLString -> URLString -> FilePath -> CommandPerform
 performQuvi relaxed pageurl videourl file = ifAnnexed file addurl geturl
   where
   	quviurl = setDownloader pageurl QuviDownloader
-  	addurl (key, _backend) = next $ cleanup quviurl file key Nothing
-	geturl = do
-		key <- Backend.URL.fromUrl quviurl Nothing
-		ifM (pure relaxed <||> Annex.getState Annex.fast)
-			( next $ cleanup quviurl file key Nothing
-			, do
-				tmp <- fromRepo $ gitAnnexTmpLocation key
+  	addurl key = next $ cleanup quviurl file key Nothing
+	geturl = next $ isJust <$> addUrlFileQuvi relaxed quviurl videourl file
+#endif
+
+#ifdef WITH_QUVI
+addUrlFileQuvi :: Bool -> URLString -> URLString -> FilePath -> Annex (Maybe Key)
+addUrlFileQuvi relaxed quviurl videourl file = do
+	key <- Backend.URL.fromUrl quviurl Nothing
+	ifM (pure relaxed <||> Annex.getState Annex.fast)
+		( do
+			cleanup' quviurl file key Nothing
+			return (Just key)
+		, do
+			{- Get the size, and use that to check
+			 - disk space. However, the size info is not
+			 - retained, because the size of a video stream
+			 - might change and we want to be able to download
+			 - it later. -}
+			sizedkey <- addSizeUrlKey videourl key
+			prepGetViaTmpChecked sizedkey Nothing $ do
+				tmp <- fromRepo $ gitAnnexTmpObjectLocation key
 				showOutput
-				ok <- Transfer.download webUUID key (Just file) Transfer.forwardRetry $ const $ do
-					liftIO $ createDirectoryIfMissing True (parentDir tmp)
-					downloadUrl [videourl] tmp
+				ok <- Transfer.notifyTransfer Transfer.Download (Just file) $
+					Transfer.download webUUID key (Just file) Transfer.forwardRetry $ const $ do
+						liftIO $ createDirectoryIfMissing True (parentDir tmp)
+						downloadUrl [videourl] tmp
 				if ok
-					then next $ cleanup quviurl file key (Just tmp)
-					else stop
-			)
+					then do
+						cleanup' quviurl file key (Just tmp)
+						return (Just key)
+					else return Nothing
+		)
 #endif
 
 perform :: Bool -> URLString -> FilePath -> CommandPerform
 perform relaxed url file = ifAnnexed file addurl geturl
   where
-	geturl = next $ addUrlFile relaxed url file
-	addurl (key, _backend)
+	geturl = next $ isJust <$> addUrlFile relaxed url file
+	addurl key
 		| relaxed = do
 			setUrlPresent key url
 			next $ return True
-		| otherwise = do
-			headers <- getHttpHeaders
-			(exists, samesize) <- Url.withUserAgent $ Url.check url headers $ keySize key
-			if exists && samesize
-				then do
-					setUrlPresent key url
-					next $ return True
-				else do
-					warning $ if exists
-						then "url does not have expected file size (use --relaxed to bypass this check) " ++ url
-						else "failed to verify url exists: " ++ url
-					stop
+		| otherwise = ifM (elem url <$> getUrls key)
+			( stop
+			, do
+				(exists, samesize) <- Url.withUrlOptions $ Url.check url (keySize key)
+				if exists && samesize
+					then do
+						setUrlPresent key url
+						next $ return True
+					else do
+						warning $ "while adding a new url to an already annexed file, " ++ if exists
+							then "url does not have expected file size (use --relaxed to bypass this check) " ++ url
+							else "failed to verify url exists: " ++ url
+						stop
+			)
 
-addUrlFile :: Bool -> URLString -> FilePath -> Annex Bool
+addUrlFile :: Bool -> URLString -> FilePath -> Annex (Maybe Key)
 addUrlFile relaxed url file = do
 	liftIO $ createDirectoryIfMissing True (parentDir file)
 	ifM (Annex.getState Annex.fast <||> pure relaxed)
@@ -145,50 +163,54 @@ addUrlFile relaxed url file = do
 			download url file
 		)
 
-download :: URLString -> FilePath -> Annex Bool
+download :: URLString -> FilePath -> Annex (Maybe Key)
 download url file = do
-	dummykey <- genkey
-	tmp <- fromRepo $ gitAnnexTmpLocation dummykey
-	showOutput
-	ifM (runtransfer dummykey tmp)
-		( do
-			backend <- chooseBackend file
-			let source = KeySource
-				{ keyFilename = file
-				, contentLocation = tmp
-				, inodeCache = Nothing
-				}
-			k <- genKey source backend
-			case k of
-				Nothing -> return False
-				Just (key, _) -> cleanup url file key (Just tmp)
-		, return False
-		)
-  where
 	{- Generate a dummy key to use for this download, before we can
 	 - examine the file and find its real key. This allows resuming
-	 - downloads, as the dummy key for a given url is stable.
-	 -
-	 - If the assistant is running, actually hits the url here,
-	 - to get the size, so it can display a pretty progress bar.
-	 -}
-	genkey = do
-		pidfile <- fromRepo gitAnnexPidFile
-		size <- ifM (liftIO $ isJust <$> checkDaemon pidfile)
+	 - downloads, as the dummy key for a given url is stable. -}
+	dummykey <- addSizeUrlKey url =<< Backend.URL.fromUrl url Nothing
+	prepGetViaTmpChecked dummykey Nothing $ do
+		tmp <- fromRepo $ gitAnnexTmpObjectLocation dummykey
+		showOutput
+		ifM (runtransfer dummykey tmp)
 			( do
-				headers <- getHttpHeaders
-				snd <$> Url.withUserAgent (Url.exists url headers)
+				backend <- chooseBackend file
+				let source = KeySource
+					{ keyFilename = file
+					, contentLocation = tmp
+					, inodeCache = Nothing
+					}
+				k <- genKey source backend
+				case k of
+					Nothing -> return Nothing
+					Just (key, _) -> do
+						cleanup' url file key (Just tmp)
+						return (Just key)
 			, return Nothing
 			)
-		Backend.URL.fromUrl url size
-  	runtransfer dummykey tmp = 
+  where
+  	runtransfer dummykey tmp =  Transfer.notifyTransfer Transfer.Download (Just file) $
 		Transfer.download webUUID dummykey (Just file) Transfer.forwardRetry $ const $ do
 			liftIO $ createDirectoryIfMissing True (parentDir tmp)
 			downloadUrl [url] tmp
-		
+
+{- Hits the url to get the size, if available.
+ -
+ - This is needed to avoid exceeding the diskreserve when downloading,
+ - and so the assistant can display a pretty progress bar.
+ -}
+addSizeUrlKey :: URLString -> Key -> Annex Key
+addSizeUrlKey url key = do
+	size <- snd <$> Url.withUrlOptions (Url.exists url)
+	return $ key { keySize = size }
 
 cleanup :: URLString -> FilePath -> Key -> Maybe FilePath -> Annex Bool
 cleanup url file key mtmp = do
+	cleanup' url file key mtmp
+	return True
+
+cleanup' :: URLString -> FilePath -> Key -> Maybe FilePath -> Annex ()
+cleanup' url file key mtmp = do
 	when (isJust mtmp) $
 		logStatus key InfoPresent
 	setUrlPresent key url
@@ -199,21 +221,20 @@ cleanup url file key mtmp = do
 		 - must already exist, so flush the queue. -}
 		Annex.Queue.flush
 	maybe noop (moveAnnex key) mtmp
-	return True
 
-nodownload :: Bool -> URLString -> FilePath -> Annex Bool
+nodownload :: Bool -> URLString -> FilePath -> Annex (Maybe Key)
 nodownload relaxed url file = do
-	headers <- getHttpHeaders
 	(exists, size) <- if relaxed
 		then pure (True, Nothing)
-		else Url.withUserAgent $ Url.exists url headers
+		else Url.withUrlOptions (Url.exists url)
 	if exists
 		then do
 			key <- Backend.URL.fromUrl url size
-			cleanup url file key Nothing
+			cleanup' url file key Nothing
+			return (Just key)
 		else do
 			warning $ "unable to access url: " ++ url
-			return False
+			return Nothing
 
 url2file :: URI -> Maybe Int -> Int -> FilePath
 url2file url pathdepth pathmax = case pathdepth of
