@@ -12,11 +12,10 @@ import Control.Exception.Extensible
 import Common.Annex
 import Command
 import qualified Git
-import qualified Git.Command
 import qualified Git.LsFiles
+import qualified Git.Branch
 import Config
 import Annex.Direct
-import Annex.Version
 import Annex.Exception
 
 def :: [Command]
@@ -24,8 +23,8 @@ def = [notBareRepo $ noDaemonRunning $
 	command "direct" paramNothing seek
 		SectionSetup "switch repository to direct mode"]
 
-seek :: [CommandSeek]
-seek = [withNothing start]
+seek :: CommandSeek
+seek = withNothing start
 
 start :: CommandStart
 start = ifM isDirect ( stop , next perform )
@@ -34,9 +33,8 @@ perform :: CommandPerform
 perform = do
 	showStart "commit" ""
 	showOutput
-	_ <- inRepo $ Git.Command.runBool
-		[ Param "commit"
-		, Param "-a"
+	_ <- inRepo $ Git.Branch.commitCommand Git.Branch.ManualCommit
+		[ Param "-a"
 		, Param "-m"
 		, Param "commit before switching to direct mode"
 		]
@@ -48,7 +46,7 @@ perform = do
 	void $ liftIO clean
 	next cleanup
   where
-	go = whenAnnexed $ \f (k, _) -> do
+	go = whenAnnexed $ \f k -> do
 		r <- toDirectGen k f
 		case r of
 			Nothing -> noop
@@ -69,5 +67,4 @@ cleanup :: CommandCleanup
 cleanup = do
 	showStart "direct" ""
 	setDirect True
-	setVersion directModeVersion
 	return True
