@@ -9,8 +9,6 @@
 
 module Command.Fsck where
 
-import System.PosixCompat.Files
-
 import Common.Annex
 import Command
 import qualified Annex
@@ -25,21 +23,17 @@ import Annex.Perms
 import Annex.Link
 import Logs.Location
 import Logs.Trust
+import Config.NumCopies
 import Annex.UUID
 import Utility.DataUnits
 import Utility.FileMode
 import Config
-import qualified Option
 import Types.Key
+import Types.CleanupActions
 import Utility.HumanTime
 import Git.FilePath
-import GitAnnex.Options hiding (fromOption)
+import Utility.PID
 
-#ifndef mingw32_HOST_OS
-import System.Posix.Process (getProcessID)
-#else
-import System.Win32.Process.Current (getCurrentProcessId)
-#endif
 import Data.Time.Clock.POSIX
 import Data.Time
 import System.Posix.Types (EpochTime)
@@ -49,48 +43,50 @@ def :: [Command]
 def = [withOptions fsckOptions $ command "fsck" paramPaths seek
 	SectionMaintenance "check for problems"]
 
-fromOption :: Option
-fromOption = Option.field ['f'] "from" paramRemote "check remote"
+fsckFromOption :: Option
+fsckFromOption = fieldOption ['f'] "from" paramRemote "check remote"
 
 startIncrementalOption :: Option
-startIncrementalOption = Option.flag ['S'] "incremental" "start an incremental fsck"
+startIncrementalOption = flagOption ['S'] "incremental" "start an incremental fsck"
 
 moreIncrementalOption :: Option
-moreIncrementalOption = Option.flag ['m'] "more" "continue an incremental fsck"
+moreIncrementalOption = flagOption ['m'] "more" "continue an incremental fsck"
 
 incrementalScheduleOption :: Option
-incrementalScheduleOption = Option.field [] "incremental-schedule" paramTime
+incrementalScheduleOption = fieldOption [] "incremental-schedule" paramTime
 	"schedule incremental fscking"
 
 fsckOptions :: [Option]
 fsckOptions = 
-	[ fromOption
+	[ fsckFromOption
 	, startIncrementalOption
 	, moreIncrementalOption
 	, incrementalScheduleOption
 	] ++ keyOptions
 
-seek :: [CommandSeek]
-seek =
-	[ withField fromOption Remote.byNameWithUUID $ \from ->
-	  withIncremental $ \i ->
-	  withKeyOptions (startKey i) $
-	  withFilesInGit $ whenAnnexed $ start from i
-	]
+seek :: CommandSeek
+seek ps = do
+	from <- getOptionField fsckFromOption Remote.byNameWithUUID
+	i <- getIncremental
+	withKeyOptions
+		(\k -> startKey i k =<< getNumCopies)
+		(withFilesInGit $ whenAnnexed $ start from i)
+		ps
 
-withIncremental :: (Incremental -> CommandSeek) -> CommandSeek
-withIncremental = withValue $ do
+getIncremental :: Annex Incremental
+getIncremental = do
 	i <- maybe (return False) (checkschedule . parseDuration)
-		=<< Annex.getField (Option.name incrementalScheduleOption)
-	starti <- Annex.getFlag (Option.name startIncrementalOption)
-	morei <- Annex.getFlag (Option.name moreIncrementalOption)
+		=<< Annex.getField (optionName incrementalScheduleOption)
+	starti <- Annex.getFlag (optionName startIncrementalOption)
+	morei <- Annex.getFlag (optionName moreIncrementalOption)
 	case (i, starti, morei) of
 		(False, False, False) -> return NonIncremental
-		(False, True, _) -> startIncremental
+		(False, True, False) -> startIncremental
 		(False ,False, True) -> ContIncremental <$> getStartTime
-		(True, _, _) ->
+		(True, False, False) ->
 			maybe startIncremental (return . ContIncremental . Just)
 				=<< getStartTime
+		_ -> error "Specify only one of --incremental, --more, or --incremental-schedule"
   where
 	startIncremental = do
 		recordStartTime
@@ -98,7 +94,7 @@ withIncremental = withValue $ do
 
 	checkschedule Nothing = error "bad --incremental-schedule value"
 	checkschedule (Just delta) = do
-		Annex.addCleanup "" $ do
+		Annex.addCleanup FsckCleanup $ do
 			v <- getStartTime
 			case v of
 				Nothing -> noop
@@ -108,16 +104,20 @@ withIncremental = withValue $ do
 						resetStartTime
 		return True
 
-start :: Maybe Remote -> Incremental -> FilePath -> (Key, Backend) -> CommandStart
-start from inc file (key, backend) = do
-	numcopies <- numCopies file
-	case from of
-		Nothing -> go $ perform key file backend numcopies
-		Just r -> go $ performRemote key file backend numcopies r
+start :: Maybe Remote -> Incremental -> FilePath -> Key -> CommandStart
+start from inc file key = do
+	v <- Backend.getBackend file key
+	case v of
+		Nothing -> stop
+		Just backend -> do
+			numcopies <- getFileNumCopies file
+			case from of
+				Nothing -> go $ perform key file backend numcopies
+				Just r -> go $ performRemote key file backend numcopies r
   where
 	go = runFsck inc file key
 
-perform :: Key -> FilePath -> Backend -> Maybe Int -> Annex Bool
+perform :: Key -> FilePath -> Backend -> NumCopies -> Annex Bool
 perform key file backend numcopies = check
 	-- order matters
 	[ fixLink key file
@@ -131,7 +131,7 @@ perform key file backend numcopies = check
 
 {- To fsck a remote, the content is retrieved to a tmp file,
  - and checked locally. -}
-performRemote :: Key -> FilePath -> Backend -> Maybe Int -> Remote -> Annex Bool
+performRemote :: Key -> FilePath -> Backend -> NumCopies -> Remote -> Annex Bool
 performRemote key file backend numcopies remote =
 	dispatch =<< Remote.hasKey remote key
   where
@@ -151,14 +151,10 @@ performRemote key file backend numcopies remote =
 		, checkKeyNumCopies key file numcopies
 		]
 	withtmp a = do
-#ifndef mingw32_HOST_OS
-		v <- liftIO getProcessID
-#else
-		v <- liftIO getCurrentProcessId
-#endif
-		t <- fromRepo gitAnnexTmpDir
+		pid <- liftIO getPID
+		t <- fromRepo gitAnnexTmpObjectDir
 		createAnnexDirectory t
-		let tmp = t </> "fsck" ++ show v ++ "." ++ keyFile key
+		let tmp = t </> "fsck" ++ show pid ++ "." ++ keyFile key
 		let cleanup = liftIO $ catchIO (removeFile tmp) (const noop)
 		cleanup
 		cleanup `after` a tmp
@@ -172,18 +168,19 @@ performRemote key file backend numcopies remote =
 			)
 	dummymeter _ = noop
 
-startKey :: Incremental -> Key -> CommandStart
-startKey inc key = case Backend.maybeLookupBackendName (Types.Key.keyBackendName key) of
-	Nothing -> stop
-	Just backend -> runFsck inc (key2file key) key $ performAll key backend
+startKey :: Incremental -> Key -> NumCopies -> CommandStart
+startKey inc key numcopies =
+	case Backend.maybeLookupBackendName (Types.Key.keyBackendName key) of
+		Nothing -> stop
+		Just backend -> runFsck inc (key2file key) key $
+			performKey key backend numcopies
 
-{- Note that numcopies cannot be checked in --all mode, since we do not
- - have associated filenames to look up in the .gitattributes file. -}
-performAll :: Key -> Backend -> Annex Bool
-performAll key backend = check
+performKey :: Key -> Backend -> NumCopies -> Annex Bool
+performKey key backend numcopies = check
 	[ verifyLocationLog key (key2file key)
 	, checkKeySize key
 	, checkBackend backend key Nothing
+	, checkKeyNumCopies key (key2file key) numcopies
 	]
 
 check :: [Annex Bool] -> Annex Bool
@@ -367,27 +364,26 @@ checkBackendOr' bad backend key file postcheck =
 				, return True
 				)
 
-checkKeyNumCopies :: Key -> FilePath -> Maybe Int -> Annex Bool
+checkKeyNumCopies :: Key -> String -> NumCopies -> Annex Bool
 checkKeyNumCopies key file numcopies = do
-	needed <- getNumCopies numcopies
 	(untrustedlocations, safelocations) <- trustPartition UnTrusted =<< Remote.keyLocations key
-	let present = length safelocations
-	if present < needed
+	let present = NumCopies (length safelocations)
+	if present < numcopies
 		then do
 			ppuuids <- Remote.prettyPrintUUIDs "untrusted" untrustedlocations
-			warning $ missingNote file present needed ppuuids
+			warning $ missingNote file present numcopies ppuuids
 			return False
 		else return True
 
-missingNote :: String -> Int -> Int -> String -> String
-missingNote file 0 _ [] = 
+missingNote :: String -> NumCopies -> NumCopies -> String -> String
+missingNote file (NumCopies 0) _ [] = 
 		"** No known copies exist of " ++ file
-missingNote file 0 _ untrusted =
+missingNote file (NumCopies 0) _ untrusted =
 		"Only these untrusted locations may have copies of " ++ file ++
 		"\n" ++ untrusted ++
 		"Back it up to trusted locations with git-annex copy."
 missingNote file present needed [] =
-		"Only " ++ show present ++ " of " ++ show needed ++ 
+		"Only " ++ show (fromNumCopies present) ++ " of " ++ show (fromNumCopies needed) ++ 
 		" trustworthy copies exist of " ++ file ++
 		"\nBack it up with git-annex copy."
 missingNote file present needed untrusted = 
@@ -418,7 +414,7 @@ badContentRemote remote key = do
 		++ Remote.name remote
 
 data Incremental = StartIncremental | ContIncremental (Maybe EpochTime) | NonIncremental
-	deriving (Eq)
+	deriving (Eq, Show)
 
 runFsck :: Incremental -> FilePath -> Key -> Annex Bool -> CommandStart
 runFsck inc file key a = ifM (needFsck inc key)
@@ -474,17 +470,23 @@ getFsckTime key = do
  -
  - To guard against time stamp damange (for example, if an annex directory
  - is copied without -a), the fsckstate file contains a time that should
- - be identical to its modification time. -}
+ - be identical to its modification time.
+ - (This is not possible to do on Windows, and so the timestamp in
+ - the file will only be equal or greater than the modification time.)
+ -}
 recordStartTime :: Annex ()
 recordStartTime = do
 	f <- fromRepo gitAnnexFsckState
 	createAnnexDirectory $ parentDir f
 	liftIO $ do
 		nukeFile f
-		h <- openFile f WriteMode
-		t <- modificationTime <$> getFileStatus f
-		hPutStr h $ showTime $ realToFrac t
-		hClose h
+		withFile f WriteMode $ \h -> do
+#ifndef mingw32_HOST_OS
+			t <- modificationTime <$> getFileStatus f
+#else
+			t <- getPOSIXTime
+#endif
+			hPutStr h $ showTime $ realToFrac t
   where
 	showTime :: POSIXTime -> String
 	showTime = show
@@ -498,11 +500,18 @@ getStartTime = do
 	f <- fromRepo gitAnnexFsckState
 	liftIO $ catchDefaultIO Nothing $ do
 		timestamp <- modificationTime <$> getFileStatus f
-		t <- readishTime <$> readFile f
-		return $ if Just (realToFrac timestamp) == t
+		let fromstatus = Just (realToFrac timestamp)
+		fromfile <- readishTime <$> readFile f
+		return $ if matchingtimestamp fromfile fromstatus
 			then Just timestamp
 			else Nothing
   where
 	readishTime :: String -> Maybe POSIXTime
 	readishTime s = utcTimeToPOSIXSeconds <$>
 		parseTime defaultTimeLocale "%s%Qs" s
+	matchingtimestamp fromfile fromstatus =
+#ifndef mingw32_HOST_OS
+		fromfile == fromstatus
+#else
+		fromfile >= fromstatus
+#endif
