@@ -31,8 +31,8 @@ def = [dontCheck repoExists $
 	command "map" paramNothing seek SectionQuery
 		"generate map of repositories"]
 
-seek :: [CommandSeek]
-seek = [withNothing start]
+seek :: CommandSeek
+seek = withNothing start
 
 start :: CommandStart
 start = do
@@ -156,7 +156,10 @@ absRepo :: Git.Repo -> Git.Repo -> Annex Git.Repo
 absRepo reference r
 	| Git.repoIsUrl reference = return $ Git.Construct.localToUrl reference r
 	| Git.repoIsUrl r = return r
-	| otherwise = liftIO $ Git.Construct.fromAbsPath =<< absPath (Git.repoPath r)
+	| otherwise = liftIO $ do
+		r' <- Git.Construct.fromAbsPath =<< absPath (Git.repoPath r)
+		r'' <- safely $ flip Annex.eval Annex.gitRepo =<< Annex.new r'
+		return (fromMaybe r' r'')
 
 {- Checks if two repos are the same. -}
 same :: Git.Repo -> Git.Repo -> Bool
@@ -190,14 +193,9 @@ tryScan :: Git.Repo -> Annex (Maybe Git.Repo)
 tryScan r
 	| Git.repoIsSsh r = sshscan
 	| Git.repoIsUrl r = return Nothing
-	| otherwise = safely $ Git.Config.read r
+	| otherwise = liftIO $ safely $ Git.Config.read r
   where
-	safely a = do
-		result <- liftIO (try a :: IO (Either SomeException Git.Repo))
-		case result of
-			Left _ -> return Nothing
-			Right r' -> return $ Just r'
-	pipedconfig cmd params = safely $
+	pipedconfig cmd params = liftIO $ safely $
 		withHandle StdoutHandle createProcessSuccess p $
 			Git.Config.hRead r
 	  where
@@ -205,7 +203,8 @@ tryScan r
 
 	configlist = Ssh.onRemote r (pipedconfig, Nothing) "configlist" [] []
 	manualconfiglist = do
-		sshparams <- Ssh.toRepo r [Param sshcmd]
+		gc <- Annex.getRemoteGitConfig r
+		sshparams <- Ssh.toRepo r gc [Param sshcmd]
 		liftIO $ pipedconfig "ssh" sshparams
 	  where
 		sshcmd = cddir ++ " && " ++
@@ -245,3 +244,10 @@ combineSame = map snd . nubBy sameuuid . map pair
   where
 	sameuuid (u1, _) (u2, _) = u1 == u2 && u1 /= NoUUID
 	pair r = (getUncachedUUID r, r)
+
+safely :: IO Git.Repo -> IO (Maybe Git.Repo)
+safely a = do
+	result <- try a :: IO (Either SomeException Git.Repo)
+	case result of
+		Left _ -> return Nothing
+		Right r' -> return $ Just r'
