@@ -14,10 +14,8 @@ import qualified Annex
 import Annex.Content
 import qualified Remote
 import Annex.UUID
+import Annex.Transfer
 import Logs.Presence
-import Logs.Transfer
-import GitAnnex.Options
-import Types.Key
 
 def :: [Command]
 def = [withOptions moveOptions $ command "move" paramPaths seek
@@ -26,16 +24,17 @@ def = [withOptions moveOptions $ command "move" paramPaths seek
 moveOptions :: [Option]
 moveOptions = fromToOptions ++ keyOptions
 
-seek :: [CommandSeek]
-seek = 
-	[ withField toOption Remote.byNameWithUUID $ \to ->
-	  withField fromOption Remote.byNameWithUUID $ \from ->
-	  withKeyOptions (startKey to from True) $
-	  withFilesInGit $ whenAnnexed $ start to from True
-	]
+seek :: CommandSeek
+seek ps = do
+	to <- getOptionField toOption Remote.byNameWithUUID
+	from <- getOptionField fromOption Remote.byNameWithUUID
+	withKeyOptions
+		(startKey to from True)
+		(withFilesInGit $ whenAnnexed $ start to from True)
+		ps
 
-start :: Maybe Remote -> Maybe Remote -> Bool -> FilePath -> (Key, Backend) -> CommandStart
-start to from move file (key, _) = start' to from move (Just file) key
+start :: Maybe Remote -> Maybe Remote -> Bool -> FilePath -> Key -> CommandStart
+start to from move file key = start' to from move (Just file) key
 
 startKey :: Maybe Remote -> Maybe Remote -> Bool -> Key -> CommandStart
 startKey to from move = start' to from move Nothing
@@ -53,17 +52,14 @@ start' to from move afile key = do
 		"--auto is not supported for move"
 
 showMoveAction :: Bool -> Key -> AssociatedFile -> Annex ()
-showMoveAction True _ (Just file) = showStart "move" file
-showMoveAction False _ (Just file) = showStart "copy" file
-showMoveAction True key Nothing = showStart "move" (key2file key)
-showMoveAction False key Nothing = showStart "copy" (key2file key)
+showMoveAction move = showStart' (if move then "move" else "copy")
 
 {- Moves (or copies) the content of an annexed file to a remote.
  -
  - If the remote already has the content, it is still removed from
  - the current repository.
  -
- - Note that unlike drop, this does not honor annex.numcopies.
+ - Note that unlike drop, this does not honor numcopies.
  - A file's content can be moved even if there are insufficient copies to
  - allow it to be dropped.
  -}
@@ -73,28 +69,38 @@ toStart dest move afile key = do
 	ishere <- inAnnex key
 	if not ishere || u == Remote.uuid dest
 		then stop -- not here, so nothing to do
-		else do
-			showMoveAction move key afile
-			next $ toPerform dest move key afile
-toPerform :: Remote -> Bool -> Key -> AssociatedFile -> CommandPerform
-toPerform dest move key afile = moveLock move key $ do
-	-- Checking the remote is expensive, so not done in the start step.
-	-- In fast mode, location tracking is assumed to be correct,
-	-- and an explicit check is not done, when copying. When moving,
-	-- it has to be done, to avoid inaverdent data loss.
+		else toStart' dest move afile key
+
+toStart' :: Remote -> Bool -> AssociatedFile -> Key -> CommandStart
+toStart' dest move afile key = do
 	fast <- Annex.getState Annex.fast
-	let fastcheck = fast && not move && not (Remote.hasKeyCheap dest)
-	isthere <- if fastcheck
-		then Right <$> expectedpresent
-		else Remote.hasKey dest key
+	if fast && not move && not (Remote.hasKeyCheap dest)
+		then ifM (expectedPresent dest key)
+			( stop
+			, go True (pure $ Right False)
+			)
+		else go False (Remote.hasKey dest key)
+  where
+	go fastcheck isthere = do
+		showMoveAction move key afile
+		next $ toPerform dest move key afile fastcheck =<< isthere
+
+expectedPresent :: Remote -> Key -> Annex Bool
+expectedPresent dest key = do
+	remotes <- Remote.keyPossibilities key
+	return $ dest `elem` remotes
+
+toPerform :: Remote -> Bool -> Key -> AssociatedFile -> Bool -> Either String Bool -> CommandPerform
+toPerform dest move key afile fastcheck isthere = moveLock move key $
 	case isthere of
 		Left err -> do
 			showNote err
 			stop
 		Right False -> do
 			showAction $ "to " ++ Remote.name dest
-			ok <- upload (Remote.uuid dest) key afile noRetry $
-				Remote.storeKey dest key afile
+			ok <- notifyTransfer Upload afile $
+				upload (Remote.uuid dest) key afile noRetry $
+					Remote.storeKey dest key afile
 			if ok
 				then do
 					Remote.logStatus dest key InfoPresent
@@ -104,7 +110,7 @@ toPerform dest move key afile = moveLock move key $ do
 						warning "This could have failed because --fast is enabled."
 					stop
 		Right True -> do
-			unlessM expectedpresent $
+			unlessM (expectedPresent dest key) $
 				Remote.logStatus dest key InfoPresent
 			finish
   where
@@ -113,9 +119,6 @@ toPerform dest move key afile = moveLock move key $ do
 			removeAnnex key
 			next $ Command.Drop.cleanupLocal key
 		| otherwise = next $ return True
-	expectedpresent = do
-		remotes <- Remote.keyPossibilities key
-		return $ dest `elem` remotes
 
 {- Moves (or copies) the content of an annexed file from a remote
  - to the current repository.
@@ -153,9 +156,10 @@ fromPerform src move key afile = moveLock move key $
 		, handle move =<< go
 		)
   where
-	go = download (Remote.uuid src) key afile noRetry $ \p -> do
-		showAction $ "from " ++ Remote.name src
-		getViaTmp key $ \t -> Remote.retrieveKeyFile src key afile t p
+	go = notifyTransfer Download afile $ 
+		download (Remote.uuid src) key afile noRetry $ \p -> do
+			showAction $ "from " ++ Remote.name src
+			getViaTmp key $ \t -> Remote.retrieveKeyFile src key afile t p
 	handle _ False = stop -- failed
 	handle False True = next $ return True -- copy complete
 	handle True True = do -- finish moving
