@@ -17,26 +17,24 @@ import qualified Git.Ref
 import qualified Git
 
 def :: [Command]
-def = [notBareRepo $ noCommit $ noMessages $
+def = [notBareRepo $ noCommit $ noMessages $ withOptions [jsonOption] $
 	command "status" paramPaths seek SectionCommon
 		"show the working tree status"]
 
-seek :: [CommandSeek]
-seek = 
-	[ withWords start
-	]
+seek :: CommandSeek
+seek = withWords start
 
 start :: [FilePath] -> CommandStart
 start [] = do
 	-- Like git status, when run without a directory, behave as if
 	-- given the path to the top of the repository.
-	cwd <- liftIO getCurrentDirectory
+	currdir <- liftIO getCurrentDirectory
 	top <- fromRepo Git.repoPath
-	next $ perform [relPathDirToFile cwd top]
-start locs = next $ perform locs
+	start' [relPathDirToFile currdir top]
+start locs = start' locs
 	
-perform :: [FilePath] -> CommandPerform
-perform locs = do
+start' :: [FilePath] -> CommandStart
+start' locs = do
 	(l, cleanup) <- inRepo $ LsFiles.modifiedOthers locs
 	getstatus <- ifM isDirect
 		( return statusDirect
@@ -44,7 +42,7 @@ perform locs = do
 		)
 	forM_ l $ \f -> maybe noop (showFileStatus f) =<< getstatus f
 	void $ liftIO cleanup
-	next $ return True
+	stop
 
 data Status 
 	= NewFile
@@ -57,7 +55,10 @@ showStatus DeletedFile = "D"
 showStatus ModifiedFile = "M"
 
 showFileStatus :: FilePath -> Status -> Annex ()
-showFileStatus f s  = liftIO $ putStrLn $ showStatus s ++ " " ++ f
+showFileStatus f s  = unlessM (showFullJSON [("status", ss), ("file", f)]) $
+	liftIO $ putStrLn $ ss ++ " " ++ f
+  where
+	ss = showStatus s
 
 statusDirect :: FilePath -> Annex (Maybe Status)
 statusDirect f = checkstatus =<< liftIO (catchMaybeIO $ getFileStatus f)
