@@ -20,13 +20,13 @@ def =
   where
 	c n = notDirect . command n paramPaths seek SectionCommon
 
-seek :: [CommandSeek]
-seek = [withFilesInGit $ whenAnnexed start]
+seek :: CommandSeek
+seek = withFilesInGit $ whenAnnexed start
 
 {- The unlock subcommand replaces the symlink with a copy of the file's
  - content. -}
-start :: FilePath -> (Key, Backend) -> CommandStart
-start file (key, _) = do
+start :: FilePath -> Key -> CommandStart
+start file key = do
 	showStart "unlock" file
 	next $ perform file key
 
@@ -36,7 +36,7 @@ perform dest key = do
 	unlessM (checkDiskSpace Nothing key 0) $ error "cannot unlock"
 
 	src <- calcRepo $ gitAnnexLocation key
-	tmpdest <- fromRepo $ gitAnnexTmpLocation key
+	tmpdest <- fromRepo $ gitAnnexTmpObjectLocation key
 	liftIO $ createDirectoryIfMissing True (parentDir tmpdest)
 	showAction "copying"
 	ifM (liftIO $ copyFileExternal src tmpdest)
