@@ -10,10 +10,6 @@
 module Command.Unused where
 
 import qualified Data.Set as S
-import qualified Data.ByteString.Lazy as L
-import Data.BloomFilter
-import Data.BloomFilter.Easy
-import Data.BloomFilter.Hash
 import Control.Monad.ST
 import qualified Data.Map as M
 
@@ -33,25 +29,26 @@ import qualified Git.DiffTree as DiffTree
 import qualified Backend
 import qualified Remote
 import qualified Annex.Branch
-import qualified Option
 import Annex.CatFile
 import Types.Key
 import Git.FilePath
+import Logs.View (is_branchView)
+import Utility.Bloom
 
 def :: [Command]
-def = [withOptions [fromOption] $ command "unused" paramNothing seek
+def = [withOptions [unusedFromOption] $ command "unused" paramNothing seek
 	SectionMaintenance "look for unused file content"]
 
-fromOption :: Option
-fromOption = Option.field ['f'] "from" paramRemote "remote to check for unused content"
+unusedFromOption :: Option
+unusedFromOption = fieldOption ['f'] "from" paramRemote "remote to check for unused content"
 
-seek :: [CommandSeek]
-seek = [withNothing start]
+seek :: CommandSeek
+seek = withNothing start
 
 {- Finds unused content in the annex. -} 
 start :: CommandStart
 start = do
-	from <- Annex.getField $ Option.name fromOption
+	from <- Annex.getField $ optionName unusedFromOption
 	let (name, action) = case from of
 		Nothing -> (".", checkUnused)
 		Just "." -> (".", checkUnused)
@@ -64,7 +61,7 @@ checkUnused :: CommandPerform
 checkUnused = chain 0
 	[ check "" unusedMsg $ findunused =<< Annex.getState Annex.fast
 	, check "bad" staleBadMsg $ staleKeysPrune gitAnnexBadDir False
-	, check "tmp" staleTmpMsg $ staleKeysPrune gitAnnexTmpDir True
+	, check "tmp" staleTmpMsg $ staleKeysPrune gitAnnexTmpObjectDir True
 	]
   where
 	findunused True = do
@@ -72,7 +69,9 @@ checkUnused = chain 0
 		return []
 	findunused False = do
 		showAction "checking for unused data"
-		excludeReferenced =<< getKeysPresent
+		-- InAnnex, not InRepository because if a direct mode
+		-- file exists, it is obviously not unused.
+		excludeReferenced =<< getKeysPresent InAnnex
 	chain _ [] = next $ return True
 	chain v (a:as) = do
 		v' <- a v
@@ -92,7 +91,7 @@ check file msg a c = do
 	l <- a
 	let unusedlist = number c l
 	unless (null l) $ showLongNote $ msg unusedlist
-	writeUnusedLog file unusedlist
+	updateUnusedLog file $ M.fromList unusedlist
 	return $ c + length l
 
 number :: Int -> [a] -> [(Int, a)]
@@ -250,7 +249,7 @@ withKeysReferenced' mdir initial a = do
 		x <- Backend.lookupFile f
 		case x of
 			Nothing -> go v fs
-			Just (k, _) -> do
+			Just k -> do
 				!v' <- a k f v
 				go v' fs
 
@@ -267,9 +266,10 @@ withKeysReferencedInGit a = do
 		map (separate (== ' ')) .
 		lines
 	nubRefs = map (Git.Ref . snd) . nubBy (\(x, _) (y, _) -> x == y)
-	ourbranchend = '/' : show Annex.Branch.name
+	ourbranchend = '/' : Git.fromRef Annex.Branch.name
 	ourbranches (_, b) = not (ourbranchend `isSuffixOf` b)
 		&& not ("refs/synced/" `isPrefixOf` b)
+		&& not (is_branchView (Git.Ref b))
 	addHead headRef refs = case headRef of
 		-- if HEAD diverges from all branches (except the branch it
 		-- points to), run the actions on staged keys (and keys
@@ -294,8 +294,8 @@ withKeysReferencedInGitRef a ref = do
 	forM_ ts $ tKey lookAtWorkingTree >=> maybe noop a
 	liftIO $ void clean
   where
-	tKey True = fmap fst <$$> Backend.lookupFile . getTopFilePath . DiffTree.file
-	tKey False = fileKey . takeFileName . encodeW8 . L.unpack <$$>
+	tKey True = Backend.lookupFile . getTopFilePath . DiffTree.file
+	tKey False = fileKey . takeFileName . decodeBS <$$>
 		catFile ref . getTopFilePath . DiffTree.file
 
 {- Looks in the specified directory for bad/tmp keys, and returns a list
@@ -326,19 +326,21 @@ data UnusedMaps = UnusedMaps
 	, unusedTmpMap :: UnusedMap
 	}
 
-{- Read unused logs once, and pass the maps to each start action. -}
 withUnusedMaps :: (UnusedMaps -> Int -> CommandStart) -> CommandSeek
 withUnusedMaps a params = do
-	unused <- readUnusedLog ""
-	unusedbad <- readUnusedLog "bad"
-	unusedtmp <- readUnusedLog "tmp"
+	unused <- readUnusedMap ""
+	unusedbad <- readUnusedMap "bad"
+	unusedtmp <- readUnusedMap "tmp"
 	let m = unused `M.union` unusedbad `M.union` unusedtmp
-	return $ map (a $ UnusedMaps unused unusedbad unusedtmp) $
+	let unusedmaps = UnusedMaps unused unusedbad unusedtmp
+	seekActions $ return $ map (a unusedmaps) $
 		concatMap (unusedSpec m) params
 
 unusedSpec :: UnusedMap -> String -> [Int]
 unusedSpec m spec
-	| spec == "all" = [fst (M.findMin m)..fst (M.findMax m)]
+	| spec == "all" = if M.null m
+		then []
+		else [fst (M.findMin m)..fst (M.findMax m)]
 	| "-" `isInfixOf` spec = range $ separate (== '-') spec
 	| otherwise = maybe badspec (: []) (readish spec)
   where
@@ -347,8 +349,8 @@ unusedSpec m spec
 		_ -> badspec
 	badspec = error $ "Expected number or range, not \"" ++ spec ++ "\""
 
-{- Start action for unused content. Finds the number in the maps, and
- - calls either of 3 actions, depending on the type of unused file. -}
+{- Seek action for unused content. Finds the number in the maps, and
+ - calls one of 3 actions, depending on the type of unused file. -}
 startUnused :: String
 	-> (Key -> CommandPerform)
 	-> (Key -> CommandPerform) 
