@@ -23,19 +23,17 @@ import Utility.Daemon (checkDaemon)
 #ifdef __ANDROID__
 import Utility.Env
 #endif
-import Init
+import Annex.Init
 import qualified Git
 import qualified Git.Config
 import qualified Git.CurrentRepo
 import qualified Annex
 import Config.Files
-import qualified Option
 import Upgrade
 import Annex.Version
 
 import Control.Concurrent
 import Control.Concurrent.STM
-import System.Process (env, std_out, std_err)
 import Network.Socket (HostName)
 import System.Environment (getArgs)
 
@@ -45,12 +43,13 @@ def = [ withOptions [listenOption] $
 	command "webapp" paramNothing seek SectionCommon "launch webapp"]
 
 listenOption :: Option
-listenOption = Option.field [] "listen" paramAddress
+listenOption = fieldOption [] "listen" paramAddress
 	"accept connections to this address"
 
-seek :: [CommandSeek]
-seek = [withField listenOption return $ \listenhost ->
-	withNothing $ start listenhost]
+seek :: CommandSeek
+seek ps = do
+	listenhost <- getOptionField listenOption return
+	withNothing (start listenhost) ps
 
 start :: Maybe HostName -> CommandStart
 start = start' True
@@ -59,40 +58,46 @@ start' :: Bool -> Maybe HostName -> CommandStart
 start' allowauto listenhost = do
 	liftIO ensureInstalled
 	ifM isInitialized 
-		( go
-		, auto
+		( maybe notinitialized (go <=< needsUpgrade) =<< getVersion
+		, if allowauto
+			then liftIO $ startNoRepo []
+			else notinitialized
 		)
 	stop
   where
-	go = do
-		cannotrun <- needsUpgrade . fromMaybe (error "no version") =<< getVersion
+	go cannotrun = do
 		browser <- fromRepo webBrowser
 		f <- liftIO . absPath =<< fromRepo gitAnnexHtmlShim
+		listenhost' <- if isJust listenhost
+			then pure listenhost
+			else annexListen <$> Annex.getGitConfig
 		ifM (checkpid <&&> checkshim f)
 			( if isJust listenhost
 				then error "The assistant is already running, so --listen cannot be used."
 				else do
 					url <- liftIO . readFile
 						=<< fromRepo gitAnnexUrlFile
-					liftIO $ openBrowser browser f url Nothing Nothing
-			, startDaemon True True Nothing cannotrun listenhost $ Just $ 
-				\origout origerr url htmlshim ->
-					if isJust listenhost
-						then maybe noop (`hPutStrLn` url) origout
-						else openBrowser browser htmlshim url origout origerr
+					liftIO $ if isJust listenhost'
+						then putStrLn url
+						else liftIO $ openBrowser browser f url Nothing Nothing
+			, do
+				startDaemon True True Nothing cannotrun listenhost' $ Just $ 
+					\origout origerr url htmlshim ->
+						if isJust listenhost'
+							then maybe noop (`hPutStrLn` url) origout
+							else openBrowser browser htmlshim url origout origerr
 			)
-	auto
-		| allowauto = liftIO $ startNoRepo []
-		| otherwise = do
-			d <- liftIO getCurrentDirectory
-			error $ "no git repository in " ++ d
 	checkpid = do
 		pidfile <- fromRepo gitAnnexPidFile
 		liftIO $ isJust <$> checkDaemon pidfile
 	checkshim f = liftIO $ doesFileExist f
+	notinitialized = do
+		g <- Annex.gitRepo
+		liftIO $ cannotStartIn (Git.repoLocation g) "repository has not been initialized by git-annex"
+		liftIO $ firstRun listenhost
 
 {- When run without a repo, start the first available listed repository in
- - the autostart file. If not, it's our first time being run! -}
+ - the autostart file. If none, it's our first time being run! -}
 startNoRepo :: CmdParams -> IO ()
 startNoRepo _ = do
 	-- FIXME should be able to reuse regular getopt, but 
@@ -101,14 +106,25 @@ startNoRepo _ = do
 	let listenhost = headMaybe $ map (snd . separate (== '=')) $ 
 		filter ("--listen=" `isPrefixOf`) args
 
-	dirs <- liftIO $ filterM doesDirectoryExist =<< readAutoStartFile
-	case dirs of
-		[] -> firstRun listenhost
-		(d:_) -> do
+	go listenhost =<< liftIO (filterM doesDirectoryExist =<< readAutoStartFile)
+  where
+	go listenhost [] = firstRun listenhost
+	go listenhost (d:ds) = do
+		v <- tryNonAsync $ do
 			setCurrentDirectory d
-			state <- Annex.new =<< Git.CurrentRepo.get
-			void $ Annex.eval state $ doCommand $
-				start' False listenhost
+			Annex.new =<< Git.CurrentRepo.get
+		case v of
+			Left e -> do
+				cannotStartIn d (show e)
+				go listenhost ds
+			Right state -> void $ Annex.eval state $ do
+				whenM (fromRepo Git.repoIsLocalBare) $
+					error $ d ++ " is a bare git repository, cannot run the webapp in it"
+				callCommandAction $
+					start' False listenhost
+
+cannotStartIn :: FilePath -> String -> IO ()
+cannotStartIn d reason = warningIO $ "unable to start webapp in repository " ++ d ++ ": " ++ reason
 
 {- Run the webapp without a repository, which prompts the user, makes one,
  - changes to it, starts the regular assistant, and redirects the
@@ -139,8 +155,9 @@ firstRun listenhost = do
 	let callback a = Just $ a v
 	runAssistant d $ do
 		startNamedThread urlrenderer $
-			webAppThread d urlrenderer True Nothing listenhost
+			webAppThread d urlrenderer True Nothing
 				(callback signaler)
+				listenhost
 				(callback mainthread)
 		waitNamedThreads
   where
@@ -153,7 +170,8 @@ firstRun listenhost = do
 			hFlush stdout
 			go
 		| otherwise = do
-			browser <- maybe Nothing webBrowser <$> Git.Config.global
+			browser <- maybe Nothing webBrowser
+				<$> catchDefaultIO Nothing Git.Config.global
 			openBrowser browser htmlshim url Nothing Nothing
 			go
 	  where
@@ -196,7 +214,16 @@ openBrowser mcmd htmlshim realurl outh errh = do
   where
 	p = case mcmd of
 		Just cmd -> proc cmd [htmlshim]
-		Nothing -> browserProc url
+		Nothing -> 
+#ifndef mingw32_HOST_OS
+			browserProc url
+#else
+			{- Windows hack to avoid using the full path,
+			 - which might contain spaces that cause problems
+			 - for browserProc. -}
+			(browserProc (takeFileName htmlshim))
+				{ cwd = Just (takeDirectory htmlshim) } 
+#endif
 #ifdef __ANDROID__
 	{- Android does not support file:// urls, but neither is
 	 - the security of the url in the process table important
