@@ -11,11 +11,11 @@ import Control.Exception.Extensible as X (IOException)
 import Data.Maybe as X
 import Data.List as X hiding (head, tail, init, last)
 import Data.String.Utils as X hiding (join)
+import Data.Monoid as X
 
 import System.FilePath as X
 import System.Directory as X
 import System.IO as X hiding (FilePath)
-import System.PosixCompat.Files as X
 #ifndef mingw32_HOST_OS
 import System.Posix.IO as X
 #endif
@@ -28,7 +28,10 @@ import Utility.Process as X
 import Utility.Path as X
 import Utility.Directory as X
 import Utility.Monad as X
+import Utility.Data as X
 import Utility.Applicative as X
 import Utility.FileSystemEncoding as X
+import Utility.PosixFiles as X
+import Utility.Network as X
 
 import Utility.PartialPrelude as X
