@@ -20,10 +20,13 @@ module Remote (
 
 	remoteTypes,
 	remoteList,
-	syncableRemote,
+	gitSyncableRemote,
 	remoteMap,
+	remoteMap',
 	uuidDescriptions,
 	byName,
+	byName',
+	byNameOnly,
 	byNameWithUUID,
 	byCost,
 	prettyPrintUUIDs,
@@ -36,17 +39,18 @@ module Remote (
 	keyPossibilities,
 	keyPossibilitiesTrusted,
 	nameToUUID,
+	nameToUUID',
 	showTriedRemotes,
 	showLocations,
 	forceTrust,
 	logStatus,
-	checkAvailable
+	checkAvailable,
+	isXMPPRemote
 ) where
 
 import qualified Data.Map as M
 import Text.JSON
 import Text.JSON.Generic
-import Data.Tuple
 import Data.Ord
 
 import Common.Annex
@@ -58,12 +62,23 @@ import Logs.Trust
 import Logs.Location hiding (logStatus)
 import Remote.List
 import Config
-import Git.Remote
+import Git.Types (RemoteName)
+import qualified Git
 
 {- Map from UUIDs of Remotes to a calculated value. -}
-remoteMap :: (Remote -> a) -> Annex (M.Map UUID a)
-remoteMap c = M.fromList . map (\r -> (uuid r, c r)) .
-	filter (\r -> uuid r /= NoUUID) <$> remoteList
+remoteMap :: (Remote -> v) -> Annex (M.Map UUID v)
+remoteMap mkv = remoteMap' mkv mkk
+  where
+	mkk r = case uuid r of
+		NoUUID -> Nothing
+		u -> Just u
+
+remoteMap' :: Ord k => (Remote -> v) -> (Remote -> Maybe k) -> Annex (M.Map k v)
+remoteMap' mkv mkk = M.fromList . mapMaybe mk <$> remoteList
+  where
+	mk r = case mkk r of
+		Nothing -> Nothing
+		Just k -> Just (k, mkv r)
 
 {- Map of UUIDs of remotes and their descriptions.
  - The names of Remotes are added to suppliment any description that has
@@ -73,9 +88,8 @@ uuidDescriptions = M.unionWith addName <$> uuidMap <*> remoteMap name
 
 addName :: String -> RemoteName -> String
 addName desc n
-	| desc == n = desc
-	| null desc = n
-	| otherwise = n ++ " (" ++ desc ++ ")"
+	| desc == n || null desc = "[" ++ n ++ "]"
+	| otherwise = desc ++ " [" ++ n ++ "]"
 
 {- When a name is specified, looks up the remote matching that name.
  - (Or it can be a UUID.) -}
@@ -89,11 +103,12 @@ byNameWithUUID = checkuuid <=< byName
   where
   	checkuuid Nothing = return Nothing
 	checkuuid (Just r)
-		| uuid r == NoUUID = do
-			let e = "cannot determine uuid for " ++ name r
+		| uuid r == NoUUID =
 			if remoteAnnexIgnore (gitconfig r)
-				then error $ e ++ " (" ++ show (remoteConfig (repo r) "ignore") ++ " is set)"
-				else error e
+				then error $ noRemoteUUIDMsg r ++
+					" (" ++ show (remoteConfig (repo r) "ignore") ++
+					" is set)"
+				else error $ noRemoteUUIDMsg r
 		| otherwise = return $ Just r
 
 byName' :: RemoteName -> Annex (Either String Remote)
@@ -104,25 +119,38 @@ byName' n = handle . filter matching <$> remoteList
 	handle (match:_) = Right match
 	matching r = n == name r || toUUID n == uuid r
 
-{- Looks up a remote by name (or by UUID, or even by description),
- - and returns its UUID. Finds even remotes that are not configured in
- - .git/config. -}
-nameToUUID :: RemoteName -> Annex UUID
-nameToUUID "." = getUUID -- special case for current repo
-nameToUUID "here" = getUUID
-nameToUUID "" = error "no remote specified"
-nameToUUID n = byName' n >>= go
+{- Only matches remote name, not UUID -}
+byNameOnly :: RemoteName -> Annex (Maybe Remote)
+byNameOnly n = headMaybe . filter matching <$> remoteList
   where
-	go (Right r) = return $ uuid r
-	go (Left e) = fromMaybe (error e) <$> bydescription
-	bydescription = do
+	matching r = n == name r
+
+noRemoteUUIDMsg :: Remote -> String
+noRemoteUUIDMsg r = "cannot determine uuid for " ++ name r
+
+{- Looks up a remote by name (or by UUID, or even by description),
+ - and returns its UUID. Finds even repositories that are not
+ - configured in .git/config. -}
+nameToUUID :: RemoteName -> Annex UUID
+nameToUUID = either error return <=< nameToUUID'
+
+nameToUUID' :: RemoteName -> Annex (Either String UUID)
+nameToUUID' "." = Right <$> getUUID -- special case for current repo
+nameToUUID' "here" = Right <$> getUUID
+nameToUUID' n = byName' n >>= go
+  where
+	go (Right r) = return $ case uuid r of
+		NoUUID -> Left $ noRemoteUUIDMsg r
+		u -> Right u
+	go (Left e) = do
 		m <- uuidMap
-		case M.lookup n $ transform swap m of
-			Just u -> return $ Just u
-			Nothing -> return $ byuuid m
-	byuuid m = M.lookup (toUUID n) $ transform double m
-	transform a = M.fromList . map a . M.toList
-	double (a, _) = (a, a)
+		return $ case M.keys (M.filter (== n) m) of
+			[u] -> Right u
+			[] -> let u = toUUID n
+				in case M.keys (M.filterWithKey (\k _ -> k == u) m) of
+					[] -> Left e
+					_ -> Right u
+			_us -> Left "Found multiple repositories with that description"
 
 {- Pretty-prints a list of UUIDs of remotes, for human display.
  -
@@ -174,8 +202,7 @@ prettyUUID u = concat <$> prettyListUUIDs [u]
 remoteFromUUID :: UUID -> Annex (Maybe Remote)
 remoteFromUUID u = ifM ((==) u <$> getUUID)
 	( return Nothing
-	, do
-		maybe tryharder (return . Just) =<< findinmap
+	, maybe tryharder (return . Just) =<< findinmap
 	)
   where
 	findinmap = M.lookup u <$> remoteMap id
@@ -279,3 +306,9 @@ byCost = map snd . sortBy (comparing fst) . M.toList . costmap
 checkAvailable :: Bool -> Remote -> IO Bool
 checkAvailable assumenetworkavailable = 
 	maybe (return assumenetworkavailable) doesDirectoryExist . localpath
+
+{- Remotes using the XMPP transport have urls like xmpp::user@host -}
+isXMPPRemote :: Remote -> Bool
+isXMPPRemote remote = Git.repoIsUrl r && "xmpp::" `isPrefixOf` Git.repoLocation r
+  where
+	r = repo remote
