@@ -21,6 +21,7 @@ import Common.Annex
 import Types.Remote
 import Types.GitConfig
 import Types.Crypto
+import Types.Creds
 import qualified Git
 import qualified Git.Command
 import qualified Git.Config
@@ -114,7 +115,7 @@ gen' r u c gc = do
 		, repo = r
 		, gitconfig = gc { remoteGitConfig = Just $ extractGitConfig r }
 		, readonly = Git.repoIsHttp r
-		, globallyAvailable = globallyAvailableCalc r
+		, availability = availabilityCalc r
 		, remotetype = remote
 	}
 	return $ Just $ encryptableRemote c
@@ -149,8 +150,8 @@ noCrypto = error "cannot use gcrypt remote without encryption enabled"
 unsupportedUrl :: Annex a
 unsupportedUrl = error "using non-ssh remote repo url with gcrypt is not supported"
 
-gCryptSetup :: Maybe UUID -> RemoteConfig -> Annex (RemoteConfig, UUID)
-gCryptSetup mu c = go $ M.lookup "gitrepo" c
+gCryptSetup :: Maybe UUID -> Maybe CredPair -> RemoteConfig -> Annex (RemoteConfig, UUID)
+gCryptSetup mu _ c = go $ M.lookup "gitrepo" c
   where
 	remotename = fromJust (M.lookup "name" c)
   	go Nothing = error "Specify gitrepo="
@@ -176,7 +177,7 @@ gCryptSetup mu c = go $ M.lookup "gitrepo" c
 		void $ inRepo $ Git.Command.runBool
 			[ Param "push"
 			, Param remotename
-			, Param $ show Annex.Branch.fullname
+			, Param $ Git.fromRef Annex.Branch.fullname
 			]
 		g <- inRepo Git.Config.reRead
 		case Git.GCrypt.remoteRepoId g (Just remotename) of
@@ -262,10 +263,14 @@ shellOrRsync r ashell arsync = case method of
  - participants, which gcrypt requires is the case, and may not be
  - depending on system configuration.
  -
- - (For shared encryption, gcrypt's default behavior is used.) -}
+ - (For shared encryption, gcrypt's default behavior is used.)
+ -
+ - Also, sets gcrypt-publish-participants to avoid unncessary gpg
+ - passphrase prompts.
+ -}
 setGcryptEncryption :: RemoteConfig -> String -> Annex ()
 setGcryptEncryption c remotename = do
-	let participants = ConfigKey $ Git.GCrypt.remoteParticipantConfigKey remotename
+	let participants = remoteconfig Git.GCrypt.remoteParticipantConfigKey
 	case extractCipher c of
 		Nothing -> noCrypto
 		Just (EncryptedCipher _ _ (KeyIds { keyIds = ks})) -> do
@@ -277,6 +282,10 @@ setGcryptEncryption c remotename = do
 				(k:_) -> setConfig signingkey k
 		Just (SharedCipher _) ->
 			unsetConfig participants
+	setConfig (remoteconfig Git.GCrypt.remotePublishParticipantConfigKey)
+		(Git.Config.boolConfig True)
+  where
+	remoteconfig n = ConfigKey $ n remotename
 
 store :: Remote -> Remote.Rsync.RsyncOpts -> (Cipher, Key) -> Key -> MeterUpdate -> Annex Bool
 store r rsyncopts (cipher, enck) k p
