@@ -22,9 +22,10 @@ import qualified Git.Config
 import qualified Git.Construct
 import qualified Git.Command
 import qualified Git.GCrypt
+import qualified Git.Types as Git
 import qualified Annex
 import Logs.Presence
-import Logs.Transfer
+import Annex.Transfer
 import Annex.UUID
 import Annex.Exception
 import qualified Annex.Content
@@ -34,9 +35,10 @@ import qualified Annex.Url as Url
 import Utility.Tmp
 import Config
 import Config.Cost
-import Init
+import Annex.Init
 import Types.Key
-import qualified Fields
+import Types.CleanupActions
+import qualified CmdLine.GitAnnexShell.Fields as Fields
 import Logs.Location
 import Utility.Metered
 #ifndef mingw32_HOST_OS
@@ -49,10 +51,10 @@ import Remote.Helper.Messages
 import qualified Remote.Helper.Ssh as Ssh
 import qualified Remote.GCrypt
 import Config.Files
+import Creds
 
 import Control.Concurrent
 import Control.Concurrent.MSampleVar
-import System.Process (std_in, std_err)
 import qualified Data.Map as M
 import Control.Exception.Extensible
 
@@ -61,7 +63,7 @@ remote = RemoteType {
 	typename = "git",
 	enumerate = list,
 	generate = gen,
-	setup = error "not supported"
+	setup = gitSetup
 }
 
 list :: Annex [Git.Repo]
@@ -79,6 +81,35 @@ list = do
 				Git.Construct.remoteNamed n $
 					Git.Construct.fromRemoteLocation url g
 
+{- Git remotes are normally set up using standard git command, not
+ - git-annex initremote and enableremote.
+ -
+ - For initremote, the git remote must already be set up, and have a uuid.
+ - Initremote simply remembers its location.
+ -
+ - enableremote simply sets up a git remote using the stored location.
+ - No attempt is made to make the remote be accessible via ssh key setup,
+ - etc.
+ -}
+gitSetup :: Maybe UUID -> Maybe CredPair -> RemoteConfig -> Annex (RemoteConfig, UUID)
+gitSetup Nothing _ c = do
+	let location = fromMaybe (error "Specify location=url") $
+		Url.parseURIRelaxed =<< M.lookup "location" c
+	g <- Annex.gitRepo
+	u <- case filter (\r -> Git.location r == Git.Url location) (Git.remotes g) of
+		[r] -> getRepoUUID r
+		[] -> error "could not find existing git remote with specified location"
+		_ -> error "found multiple git remotes with specified location"
+	return (c, u)
+gitSetup (Just u) _ c = do
+	inRepo $ Git.Command.run
+		[ Param "remote"
+		, Param "add"
+		, Param $ fromMaybe (error "no name") (M.lookup "name" c)
+		, Param $ fromMaybe (error "no location") (M.lookup "location" c)
+		]
+	return (c, u)
+
 {- It's assumed to be cheap to read the config of non-URL remotes, so this is
  - done each time git-annex is run in a way that uses remotes.
  -
@@ -86,10 +117,9 @@ list = do
  - cached UUID value. -}
 configRead :: Git.Repo -> Annex Git.Repo
 configRead r = do
-	g <- fromRepo id
-	let c = extractRemoteGitConfig g (Git.repoDescribe r)
+	gc <- Annex.getRemoteGitConfig r
 	u <- getRepoUUID r
-	case (repoCheap r, remoteAnnexIgnore c, u) of
+	case (repoCheap r, remoteAnnexIgnore gc, u) of
 		(_, True, _) -> return r
 		(True, _, _) -> tryGitConfigRead r
 		(False, _, NoUUID) -> tryGitConfigRead r
@@ -111,7 +141,7 @@ gen r u c gc
 			, retrieveKeyFile = copyFromRemote new
 			, retrieveKeyFileCheap = copyFromRemoteCheap new
 			, removeKey = dropKey new
-			, hasKey = inAnnex r
+			, hasKey = inAnnex new
 			, hasKeyCheap = repoCheap r
 			, whereisKey = Nothing
 			, remoteFsck = if Git.repoIsUrl r
@@ -126,7 +156,7 @@ gen r u c gc
 			, gitconfig = gc
 				{ remoteGitConfig = Just $ extractGitConfig r }
 			, readonly = Git.repoIsHttp r
-			, globallyAvailable = globallyAvailableCalc r
+			, availability = availabilityCalc r
 			, remotetype = remote
 			}
 
@@ -144,7 +174,7 @@ repoAvail r
 				else return True
 	| Git.repoIsUrl r = return True
 	| Git.repoIsLocalUnknown r = return False
-	| otherwise = liftIO $ catchBoolIO $ onLocal r $ return True
+	| otherwise = liftIO $ isJust <$> catchMaybeIO (Git.Config.read r)
 
 {- Tries to read the config for a specified remote, updates state, and
  - returns the updated repo. -}
@@ -158,22 +188,14 @@ tryGitConfigRead r
 				| haveconfig r' -> return r'
 				| otherwise -> configlist_failed
 			Left _ -> configlist_failed
-	| Git.repoIsHttp r = do
-		headers <- getHttpHeaders
-		store $ geturlconfig headers
+	| Git.repoIsHttp r = store geturlconfig
 	| Git.GCrypt.isEncrypted r = handlegcrypt =<< getConfigMaybe (remoteConfig r "uuid")
 	| Git.repoIsUrl r = return r
-	| otherwise = store $ safely $ onLocal r $ do 
-		ensureInitialized
-		Annex.getState Annex.repo
+	| otherwise = store $ liftIO $ 
+		readlocalannexconfig `catchNonAsync` (const $ return r)
   where
 	haveconfig = not . M.null . Git.config
 
-	-- Reading config can fail due to IO error or
-	-- for other reasons; catch all possible exceptions.
-	safely a = either (const $ return r) return
-			=<< liftIO (try a :: IO (Either SomeException Git.Repo))
-
 	pipedconfig cmd params = do
 		v <- Git.Config.fromPipe r cmd params
 		case v of
@@ -185,19 +207,24 @@ tryGitConfigRead r
 				return $ Right r'
 			Left l -> return $ Left l
 
-	geturlconfig headers = do
-		ua <- Url.getUserAgent
+	geturlconfig = do
+		uo <- Url.getUrlOptions
 		v <- liftIO $ withTmpFile "git-annex.tmp" $ \tmpfile h -> do
 			hClose h
-			ifM (Url.downloadQuiet (Git.repoLocation r ++ "/config") headers [] tmpfile ua)
+			ifM (Url.downloadQuiet (Git.repoLocation r ++ "/config") tmpfile uo)
 				( pipedconfig "git" [Param "config", Param "--null", Param "--list", Param "--file", File tmpfile]
 				, return $ Left undefined
 				)
 		case v of
 			Left _ -> do
-				set_ignore "not usable by git-annex"
+				set_ignore "not usable by git-annex" False
 				return r
-			Right r' -> return r'
+			Right r' -> do
+				-- Cache when http remote is not bare for
+				-- optimisation.
+				unless (Git.Config.isBare r') $
+					setremote "annex-bare" (Git.Config.boolConfig False)
+				return r'
 
 	store = observe $ \r' -> do
 		g <- gitRepo
@@ -218,16 +245,25 @@ tryGitConfigRead r
 	configlist_failed = case Git.remoteName r of
 		Nothing -> return r
 		Just n -> do
-			whenM (inRepo $ Git.Command.runBool [Param "fetch", Param "--quiet", Param n]) $
-				set_ignore "does not have git-annex installed"
+			whenM (inRepo $ Git.Command.runBool [Param "fetch", Param "--quiet", Param n]) $ do
+				set_ignore "does not have git-annex installed" True
 			return r
 	
-	set_ignore msg = case Git.remoteName r of
+	set_ignore msg longmessage = do
+		let k = "annex-ignore"
+		case Git.remoteName r of
+			Nothing -> noop
+			Just n -> do
+				warning $ "Remote " ++ n ++ " " ++ msg ++ "; setting " ++ k
+				when longmessage $
+					warning $ "This could be a problem with the git-annex installation on the remote. Please make sure that git-annex-shell is available in PATH when you ssh into the remote. Once you have fixed the git-annex installation, run: git config remote." ++ n ++ "." ++ k ++ " false"
+		setremote k (Git.Config.boolConfig True)
+	
+	setremote k v = case Git.remoteName r of
 		Nothing -> noop
 		Just n -> do
-			let k = "remote." ++ n ++ ".annex-ignore"
-			warning $ "Remote " ++ n ++ " " ++ msg ++ "; setting " ++ k
-			inRepo $ Git.Command.run [Param "config", Param k, Param "true"]
+			let k' = "remote." ++ n ++ "." ++ k
+			inRepo $ Git.Command.run [Param "config", Param k', Param v]
 		
 	handlegcrypt Nothing = return r
 	handlegcrypt (Just _cacheduuid) = do
@@ -238,45 +274,61 @@ tryGitConfigRead r
 			Just v -> store $ liftIO $ setUUID r $
 				genUUIDInNameSpace gCryptNameSpace v
 
+	{- The local repo may not yet be initialized, so try to initialize
+	 - it if allowed. However, if that fails, still return the read
+	 - git config. -}
+	readlocalannexconfig = do
+		s <- Annex.new r
+		Annex.eval s $ do
+			Annex.BranchState.disableUpdate
+			void $ tryAnnex $ ensureInitialized
+			Annex.getState Annex.repo
+
 {- Checks if a given remote has the content for a key inAnnex.
  - If the remote cannot be accessed, or if it cannot determine
  - whether it has the content, returns a Left error message.
  -}
-inAnnex :: Git.Repo -> Key -> Annex (Either String Bool)
-inAnnex r key
-	| Git.repoIsHttp r = checkhttp =<< getHttpHeaders
+inAnnex :: Remote -> Key -> Annex (Either String Bool)
+inAnnex rmt key
+	| Git.repoIsHttp r = checkhttp
 	| Git.repoIsUrl r = checkremote
 	| otherwise = checklocal
   where
-	checkhttp headers = do
+  	r = repo rmt
+	checkhttp = do
 		showChecking r
-		ifM (anyM (\u -> Url.withUserAgent $ Url.checkBoth u headers (keySize key)) (keyUrls r key))
+		ifM (Url.withUrlOptions $ \uo -> anyM (\u -> Url.checkBoth u (keySize key) uo) (keyUrls rmt key))
 			( return $ Right True
 			, return $ Left "not found"
 			)
 	checkremote = Ssh.inAnnex r key
 	checklocal = guardUsable r (cantCheck r) $ dispatch <$> check
 	  where
-		check = liftIO $ catchMsgIO $ onLocal r $
-			Annex.Content.inAnnexSafe key
+		check = either (Left . show) Right 
+			<$> tryAnnex (onLocal rmt $ Annex.Content.inAnnexSafe key)
 		dispatch (Left e) = Left e
 		dispatch (Right (Just b)) = Right b
 		dispatch (Right Nothing) = cantCheck r
 
-keyUrls :: Git.Repo -> Key -> [String]
-keyUrls r key = map tourl locs
+keyUrls :: Remote -> Key -> [String]
+keyUrls r key = map tourl locs'
   where
-	tourl l = Git.repoLocation r ++ "/" ++ l
+	tourl l = Git.repoLocation (repo r) ++ "/" ++ l
+	-- If the remote is known to not be bare, try the hash locations
+	-- used for non-bare repos first, as an optimisation.
+	locs
+		| remoteAnnexBare (gitconfig r) == Just False = reverse (annexLocations key)
+		| otherwise = annexLocations key
 #ifndef mingw32_HOST_OS
-	locs = annexLocations key
+	locs' = locs
 #else
-	locs = map (replace "\\" "/") (annexLocations key)
+	locs' = map (replace "\\" "/") locs
 #endif
 
 dropKey :: Remote -> Key -> Annex Bool
 dropKey r key
 	| not $ Git.repoIsUrl (repo r) =
-		guardUsable (repo r) False $ commitOnCleanup r $ liftIO $ onLocal (repo r) $ do
+		guardUsable (repo r) False $ commitOnCleanup r $ onLocal r $ do
 			ensureInitialized
 			whenM (Annex.Content.inAnnex key) $ do
 				Annex.Content.lockContent key $
@@ -293,23 +345,23 @@ copyFromRemote r key file dest _p = copyFromRemote' r key file dest
 copyFromRemote' :: Remote -> Key -> AssociatedFile -> FilePath -> Annex Bool
 copyFromRemote' r key file dest
 	| not $ Git.repoIsUrl (repo r) = guardUsable (repo r) False $ do
-		let params = Ssh.rsyncParams r
+		params <- Ssh.rsyncParams r Download
 		u <- getUUID
 		-- run copy from perspective of remote
-		liftIO $ onLocal (repo r) $ do
+		onLocal r $ do
 			ensureInitialized
 			v <- Annex.Content.prepSendAnnex key
 			case v of
 				Nothing -> return False
 				Just (object, checksuccess) ->
-					upload u key file noRetry
+					runTransfer (Transfer Download u key) file noRetry
 						(rsyncOrCopyFile params object dest)
 						<&&> checksuccess
 	| Git.repoIsSsh (repo r) = feedprogressback $ \feeder -> do
 		direct <- isDirect
 		Ssh.rsyncHelper (Just feeder) 
 			=<< Ssh.rsyncParamsRemote direct r Download key dest file
-	| Git.repoIsHttp (repo r) = Annex.Content.downloadUrl (keyUrls (repo r) key) dest
+	| Git.repoIsHttp (repo r) = Annex.Content.downloadUrl (keyUrls r key) dest
 	| otherwise = error "copying from non-ssh, non-http remote not supported"
   where
 	{- Feed local rsync's progress info back to the remote,
@@ -392,14 +444,14 @@ copyToRemote r key file p
 		-- the remote's Annex, but it needs access to the current
 		-- Annex monad's state.
 		checksuccessio <- Annex.withCurrentState checksuccess
-		let params = Ssh.rsyncParams r
+		params <- Ssh.rsyncParams r Upload
 		u <- getUUID
 		-- run copy from perspective of remote
-		liftIO $ onLocal (repo r) $ ifM (Annex.Content.inAnnex key)
+		onLocal r $ ifM (Annex.Content.inAnnex key)
 			( return True
 			, do
 				ensureInitialized
-				download u key file noRetry $ const $
+				runTransfer (Transfer Download u key) file noRetry $ const $
 					Annex.Content.saveState True `after`
 						Annex.Content.getViaTmpChecked (liftIO checksuccessio) key
 							(\d -> rsyncOrCopyFile params object d p)
@@ -414,29 +466,50 @@ fsckOnRemote r params
 			Just (c, ps) -> batchCommand c ps
 	| otherwise = return $ do
 		program <- readProgramFile
-		env <- getEnvironment
 		r' <- Git.Config.read r
-		let env' =
+		environ <- getEnvironment
+		let environ' = addEntries 
 			[ ("GIT_WORK_TREE", Git.repoPath r')
 			, ("GIT_DIR", Git.localGitDir r')
-			] ++ env
-		batchCommandEnv program (Param "fsck" : params) (Just env')
+			] environ
+		batchCommandEnv program (Param "fsck" : params) $ Just environ'
 
 {- The passed repair action is run in the Annex monad of the remote. -}
 repairRemote :: Git.Repo -> Annex Bool -> Annex (IO Bool)
-repairRemote r a = return $ Remote.Git.onLocal r a
-
-{- Runs an action on a local repository inexpensively, by making an annex
- - monad using that repository. -}
-onLocal :: Git.Repo -> Annex a -> IO a
-onLocal r a = do
+repairRemote r a = return $ do
 	s <- Annex.new r
 	Annex.eval s $ do
-		-- No need to update the branch; its data is not used
-		-- for anything onLocal is used to do.
 		Annex.BranchState.disableUpdate
+		ensureInitialized
 		a
 
+{- Runs an action from the perspective of a local remote.
+ -
+ - The AnnexState is cached for speed and to avoid resource leaks.
+ -
+ - The repository's git-annex branch is not updated, as an optimisation.
+ - No caller of onLocal can query data from the branch and be ensured
+ - it gets a current value. Caller of onLocal can make changes to
+ - the branch, however.
+ -}
+onLocal :: Remote -> Annex a -> Annex a
+onLocal r a = do
+	m <- Annex.getState Annex.remoteannexstate
+	case M.lookup (uuid r) m of
+		Nothing -> do
+			st <- liftIO $ Annex.new (repo r)
+			go st $ do
+				Annex.BranchState.disableUpdate	
+				a
+		Just st -> go st a
+  where
+	cache st = Annex.changeState $ \s -> s
+		{ Annex.remoteannexstate = M.insert (uuid r) st (Annex.remoteannexstate s) }
+	go st a' = do
+		(ret, st') <- liftIO $ Annex.run st a'
+		cache st'
+		return ret
+
 {- Copys a file with rsync unless both locations are on the same
  - filesystem. Then cp could be faster. -}
 rsyncOrCopyFile :: [CommandParam] -> FilePath -> FilePath -> MeterUpdate -> Annex Bool
@@ -471,9 +544,9 @@ rsyncOrCopyFile rsyncparams src dest p =
 commitOnCleanup :: Remote -> Annex a -> Annex a
 commitOnCleanup r a = go `after` a
   where
-	go = Annex.addCleanup (Git.repoLocation $ repo r) cleanup
+	go = Annex.addCleanup (RemoteCleanup $ uuid r) cleanup
 	cleanup
-		| not $ Git.repoIsUrl (repo r) = liftIO $ onLocal (repo r) $
+		| not $ Git.repoIsUrl (repo r) = onLocal r $
 			doQuietSideAction $
 				Annex.Branch.commit "update"
 		| otherwise = void $ do
