@@ -9,7 +9,6 @@ module Remote.Glacier (remote, jobList) where
 
 import qualified Data.Map as M
 import qualified Data.Text as T
-import System.Environment
 
 import Common.Annex
 import Types.Remote
@@ -26,8 +25,7 @@ import Utility.Metered
 import qualified Annex
 import Annex.Content
 import Annex.UUID
-
-import System.Process
+import Utility.Env
 
 type Vault = String
 type Archive = FilePath
@@ -66,22 +64,23 @@ gen r u c gc = new <$> remoteCost gc veryExpensiveRemoteCost
 			gitconfig = gc,
 			localpath = Nothing,
 			readonly = False,
-			globallyAvailable = True,
+			availability = GloballyAvailable,
 			remotetype = remote
 		}
 
-glacierSetup :: Maybe UUID -> RemoteConfig -> Annex (RemoteConfig, UUID)
-glacierSetup mu c = do
+glacierSetup :: Maybe UUID -> Maybe CredPair -> RemoteConfig -> Annex (RemoteConfig, UUID)
+glacierSetup mu mcreds c = do
 	u <- maybe (liftIO genUUID) return mu
-	glacierSetup' u c
-glacierSetup' :: UUID -> RemoteConfig -> Annex (RemoteConfig, UUID)
-glacierSetup' u c = do
+	c' <- setRemoteCredPair c (AWS.creds u) mcreds
+	glacierSetup' (isJust mu) u c'
+glacierSetup' :: Bool -> UUID -> RemoteConfig -> Annex (RemoteConfig, UUID)
+glacierSetup' enabling u c = do
 	c' <- encryptionSetup c
 	let fullconfig = c' `M.union` defaults
-	genVault fullconfig u
+	unless enabling $
+		genVault fullconfig u
 	gitConfigSpecialRemote u fullconfig "glacier" "true"
-	c'' <- setRemoteCredPair fullconfig (AWS.creds u)
-	return (c'', u)
+	return (fullconfig, u)
   where
 	remotename = fromJust (M.lookup "name" c)
 	defvault = remotename ++ "-" ++ fromUUID u
@@ -195,7 +194,7 @@ checkPresent r k = do
 					else return $ Right False
 			Left err -> return $ Left err
 
-	params =
+	params = glacierParams (config r)
 		[ Param "archive"
 		, Param "checkpresent"
 		, Param $ getVault $ config r
@@ -224,7 +223,8 @@ glacierParams :: RemoteConfig -> [CommandParam] -> [CommandParam]
 glacierParams c params = datacenter:params
   where
 	datacenter = Param $ "--region=" ++
-		fromJust (M.lookup "datacenter" c)
+		fromMaybe (error "Missing datacenter configuration")
+			(M.lookup "datacenter" c)
 
 glacierEnv :: RemoteConfig -> UUID -> Annex (Maybe [(String, String)])
 glacierEnv c u = go =<< getRemoteCredPairFor "glacier" c creds
@@ -232,20 +232,20 @@ glacierEnv c u = go =<< getRemoteCredPairFor "glacier" c creds
 	go Nothing = return Nothing
 	go (Just (user, pass)) = do
 		e <- liftIO getEnvironment
-		return $ Just $ (uk, user):(pk, pass):e
+		return $ Just $ addEntries [(uk, user), (pk, pass)] e
 
 	creds = AWS.creds u
 	(uk, pk) = credPairEnvironment creds
 
 getVault :: RemoteConfig -> Vault
-getVault = fromJust . M.lookup "vault"
+getVault = fromMaybe (error "Missing vault configuration") 
+	. M.lookup "vault"
 
 archive :: Remote -> Key -> Archive
 archive r k = fileprefix ++ key2file k
   where
 	fileprefix = M.findWithDefault "" "fileprefix" $ config r
 
--- glacier vault create will succeed even if the vault already exists.
 genVault :: RemoteConfig -> UUID -> Annex ()
 genVault c u = unlessM (runGlacier c u params) $
 	error "Failed creating glacier vault."
