@@ -13,10 +13,13 @@ import qualified Data.Map as M
 
 import Common.Annex
 import Types.Remote
+import Types.CleanupActions
 import qualified Annex
-import Annex.LockPool
+import Annex.LockFile
 #ifndef mingw32_HOST_OS
 import Annex.Perms
+#else
+import Utility.WinLock
 #endif
 
 {- Modifies a remote's access functions to first run the
@@ -45,7 +48,7 @@ runHooks :: Remote -> Maybe String -> Maybe String -> Annex a -> Annex a
 runHooks r starthook stophook a = do
 	dir <- fromRepo gitAnnexRemotesDir
 	let lck = dir </> remoteid ++ ".lck"
-	whenM (notElem lck . M.keys <$> getPool) $ do
+	whenM (notElem lck . M.keys <$> getLockPool) $ do
 		liftIO $ createDirectoryIfMissing True dir
 		firstrun lck
 	a
@@ -60,7 +63,7 @@ runHooks r starthook stophook a = do
 		-- of it from running the stophook. If another
 		-- instance is shutting down right now, this
 		-- will block waiting for its exclusive lock to clear.
-		lockFile lck
+		lockFileShared lck
 
 		-- The starthook is run even if some other git-annex
 		-- is already running, and ran it before.
@@ -72,14 +75,14 @@ runHooks r starthook stophook a = do
 		-- So, requiring idempotency is the right approach.
 		run starthook
 
-		Annex.addCleanup (remoteid ++ "-stop-command") $ runstop lck
-#ifndef mingw32_HOST_OS
+		Annex.addCleanup (StopHook $ uuid r) $ runstop lck
 	runstop lck = do
 		-- Drop any shared lock we have, and take an
 		-- exclusive lock, without blocking. If the lock
 		-- succeeds, we're the only process using this remote,
 		-- so can stop it.
 		unlockFile lck
+#ifndef mingw32_HOST_OS
 		mode <- annexFileMode
 		fd <- liftIO $ noUmask mode $
 			openFd lck ReadWrite (Just mode) defaultFileFlags
@@ -90,5 +93,10 @@ runHooks r starthook stophook a = do
 			Right _ -> run stophook
 		liftIO $ closeFd fd
 #else
-	runstop _lck = run stophook
+		v <- liftIO $ lockExclusive lck
+		case v of
+			Nothing -> noop
+			Just lockhandle -> do
+				run stophook
+				liftIO $ dropLock lockhandle
 #endif
