@@ -8,28 +8,27 @@
 module Remote.Helper.Ssh where
 
 import Common.Annex
+import qualified Annex
 import qualified Git
 import qualified Git.Url
 import Annex.UUID
 import Annex.Ssh
-import Fields (Field, fieldName)
-import qualified Fields
-import Types.GitConfig
+import CmdLine.GitAnnexShell.Fields (Field, fieldName)
+import qualified CmdLine.GitAnnexShell.Fields as Fields
 import Types.Key
 import Remote.Helper.Messages
 import Utility.Metered
 import Utility.Rsync
 import Types.Remote
 import Logs.Transfer
+import Config
 
 {- Generates parameters to ssh to a repository's host and run a command.
  - Caller is responsible for doing any neccessary shellEscaping of the
  - passed command. -}
-toRepo :: Git.Repo -> [CommandParam] -> Annex [CommandParam]
-toRepo r sshcmd = do
-	g <- fromRepo id
-	let c = extractRemoteGitConfig g (Git.repoDescribe r)
-	let opts = map Param $ remoteAnnexSshOptions c
+toRepo :: Git.Repo -> RemoteGitConfig -> [CommandParam] -> Annex [CommandParam]
+toRepo r gc sshcmd = do
+	let opts = map Param $ remoteAnnexSshOptions gc
 	let host = fromMaybe (error "bad ssh url") $ Git.Url.hostuser r
 	params <- sshCachingOptions (host, Git.Url.port r) opts
 	return $ params ++ Param host : sshcmd
@@ -40,16 +39,18 @@ git_annex_shell :: Git.Repo -> String -> [CommandParam] -> [(Field, String)] ->
 git_annex_shell r command params fields
 	| not $ Git.repoIsUrl r = return $ Just (shellcmd, shellopts ++ fieldopts)
 	| Git.repoIsSsh r = do
+		gc <- Annex.getRemoteGitConfig r
 		u <- getRepoUUID r
-		sshparams <- toRepo r [Param $ sshcmd u ]
+		sshparams <- toRepo r gc [Param $ sshcmd u gc]
 		return $ Just ("ssh", sshparams)
 	| otherwise = return Nothing
   where
 	dir = Git.repoPath r
 	shellcmd = "git-annex-shell"
 	shellopts = Param command : File dir : params
-	sshcmd u = unwords $
-		shellcmd : map shellEscape (toCommand shellopts) ++
+	sshcmd u gc = unwords $
+		fromMaybe shellcmd (remoteAnnexShell gc)
+			: map shellEscape (toCommand shellopts) ++
 		uuidcheck u ++
 		map shellEscape (toCommand fieldopts)
 	uuidcheck NoUUID = []
@@ -122,7 +123,7 @@ rsyncParamsRemote direct r direction key file afile = do
 		fields
 	-- Convert the ssh command into rsync command line.
 	let eparam = rsyncShell (Param shellcmd:shellparams)
-	let o = rsyncParams r
+	o <- rsyncParams r direction
 	return $ if direction == Download
 		then o ++ rsyncopts eparam dummy (File file)
 		else o ++ rsyncopts eparam (File file) dummy
@@ -140,7 +141,21 @@ rsyncParamsRemote direct r direction key file afile = do
 	dummy = Param "dummy:"
 
 -- --inplace to resume partial files
-rsyncParams :: Remote -> [CommandParam]
-rsyncParams r = Params "--progress --inplace" :
-	map Param (remoteAnnexRsyncOptions $ gitconfig r)
-
+--
+-- Only use --perms when not on a crippled file system, as rsync
+-- will fail trying to restore file perms onto a filesystem that does not
+-- support them.
+rsyncParams :: Remote -> Direction -> Annex [CommandParam]
+rsyncParams r direction = do
+	crippled <- crippledFileSystem
+	return $ map Param $ catMaybes
+		[ Just "--progress"
+		, Just "--inplace"
+		, if crippled then Nothing else Just "--perms"
+		] 
+		++ remoteAnnexRsyncOptions gc ++ dps
+  where
+	dps
+		| direction == Download = remoteAnnexRsyncDownloadOptions gc
+		| otherwise = remoteAnnexRsyncUploadOptions gc
+	gc = gitconfig r
