@@ -9,11 +9,11 @@ module Remote.Hook (remote) where
 
 import qualified Data.ByteString.Lazy as L
 import qualified Data.Map as M
-import System.Environment
 
 import Common.Annex
 import Types.Remote
 import Types.Key
+import Types.Creds
 import qualified Git
 import Config
 import Config.Cost
@@ -23,6 +23,7 @@ import Remote.Helper.Special
 import Remote.Helper.Encryptable
 import Crypto
 import Utility.Metered
+import Utility.Env
 
 type Action = String
 type HookName = String
@@ -59,14 +60,14 @@ gen r u c gc = do
 			repo = r,
 			gitconfig = gc,
 			readonly = False,
-			globallyAvailable = False,
+			availability = GloballyAvailable,
 			remotetype = remote
 		}
   where
-	hooktype = fromMaybe (error "missing hooktype") $ remoteAnnexHookType gc	
+	hooktype = fromMaybe (error "missing hooktype") $ remoteAnnexHookType gc
 
-hookSetup :: Maybe UUID -> RemoteConfig -> Annex (RemoteConfig, UUID)
-hookSetup mu c = do
+hookSetup :: Maybe UUID -> Maybe CredPair -> RemoteConfig -> Annex (RemoteConfig, UUID)
+hookSetup mu _ c = do
 	u <- maybe (liftIO genUUID) return mu
 	let hooktype = fromMaybe (error "Specify hooktype=") $
 		M.lookup "hooktype" c
@@ -77,17 +78,16 @@ hookSetup mu c = do
 hookEnv :: Action -> Key -> Maybe FilePath -> IO (Maybe [(String, String)])
 hookEnv action k f = Just <$> mergeenv (fileenv f ++ keyenv)
   where
-	mergeenv l = M.toList . M.union (M.fromList l) 
-		<$> M.fromList <$> getEnvironment
-	env s v = ("ANNEX_" ++ s, v)
+	mergeenv l = addEntries l <$> getEnvironment
+	envvar s v = ("ANNEX_" ++ s, v)
 	keyenv = catMaybes
-		[ Just $ env "KEY" (key2file k)
-		, Just $ env "ACTION" action
-		, env "HASH_1" <$> headMaybe hashbits
-		, env "HASH_2" <$> headMaybe (drop 1 hashbits)
+		[ Just $ envvar "KEY" (key2file k)
+		, Just $ envvar "ACTION" action
+		, envvar "HASH_1" <$> headMaybe hashbits
+		, envvar "HASH_2" <$> headMaybe (drop 1 hashbits)
 		]
 	fileenv Nothing = []
-	fileenv (Just file) =  [env "FILE" file]
+	fileenv (Just file) =  [envvar "FILE" file]
 	hashbits = map takeDirectory $ splitPath $ hashDirMixed k
 
 lookupHook :: HookName -> Action -> Annex (Maybe String)
@@ -155,5 +155,5 @@ checkPresent r h k = do
 	findkey s = key2file k `elem` lines s
 	check Nothing = error $ action ++ " hook misconfigured"
 	check (Just hook) = do
-		env <- hookEnv action k Nothing
-		findkey <$> readProcessEnv "sh" ["-c", hook] env
+		environ <- hookEnv action k Nothing
+		findkey <$> readProcessEnv "sh" ["-c", hook] environ
