@@ -12,7 +12,8 @@ import Network.AWS.S3Object hiding (getStorageClass)
 import Network.AWS.S3Bucket hiding (size)
 import Network.AWS.AWSResult
 import qualified Data.Text as T
-import qualified Data.ByteString.Lazy.Char8 as L
+import qualified Data.Text.Encoding as T
+import qualified Data.ByteString.Lazy as L
 import qualified Data.Map as M
 import Data.Char
 import Network.Socket (HostName)
@@ -69,14 +70,15 @@ gen r u c gc = new <$> remoteCost gc expensiveRemoteCost
 			gitconfig = gc,
 			localpath = Nothing,
 			readonly = False,
-			globallyAvailable = True,
+			availability = GloballyAvailable,
 			remotetype = remote
 		}
 
-s3Setup :: Maybe UUID -> RemoteConfig -> Annex (RemoteConfig, UUID)
-s3Setup mu c = do
+s3Setup :: Maybe UUID -> Maybe CredPair -> RemoteConfig -> Annex (RemoteConfig, UUID)
+s3Setup mu mcreds c = do
 	u <- maybe (liftIO genUUID) return mu
-	s3Setup' u c
+	c' <- setRemoteCredPair c (AWS.creds u) mcreds
+	s3Setup' u c'
 s3Setup' :: UUID -> RemoteConfig -> Annex (RemoteConfig, UUID)
 s3Setup' u c = if isIA c then archiveorg else defaulthost
   where
@@ -92,8 +94,7 @@ s3Setup' u c = if isIA c then archiveorg else defaulthost
 		
 	use fullconfig = do
 		gitConfigSpecialRemote u fullconfig "s3" "true"
-		c' <- setRemoteCredPair fullconfig (AWS.creds u)
-		return (c', u)
+		return (fullconfig, u)
 
 	defaulthost = do
 		c' <- encryptionSetup c
@@ -254,20 +255,28 @@ iaMunge = (>>= munge)
 		| isSpace c = []
 		| otherwise = "&" ++ show (ord c) ++ ";"
 
+{- Generate the bucket if it does not already exist, including creating the
+ - UUID file within the bucket.
+ -
+ - To check if the bucket exists, ask for its location. However, some ACLs
+ - can allow read/write to buckets, but not querying location, so first
+ - check if the UUID file already exists and we can skip doing anything.
+ -}
 genBucket :: RemoteConfig -> UUID -> Annex ()
 genBucket c u = do
 	conn <- s3ConnectionRequired c u
 	showAction "checking bucket"
-	loc <- liftIO $ getBucketLocation conn bucket 
-	case loc of
-		Right _ -> writeUUIDFile c u
-		Left err@(NetworkError _) -> s3Error err
-		Left (AWSError _ _) -> do
-			showAction $ "creating bucket in " ++ datacenter
-			res <- liftIO $ createBucketIn conn bucket datacenter
-			case res of
-				Right _ -> writeUUIDFile c u
-				Left err -> s3Error err
+	unlessM ((== Right True) <$> checkUUIDFile c u conn) $ do
+		loc <- liftIO $ getBucketLocation conn bucket
+		case loc of
+			Right _ -> writeUUIDFile c u
+			Left err@(NetworkError _) -> s3Error err
+			Left (AWSError _ _) -> do
+				showAction $ "creating bucket in " ++ datacenter
+				res <- liftIO $ createBucketIn conn bucket datacenter
+				case res of
+					Right _ -> writeUUIDFile c u
+					Left err -> s3Error err
   where
 	bucket = fromJust $ getBucket c
 	datacenter = fromJust $ M.lookup "datacenter" c
@@ -283,20 +292,38 @@ genBucket c u = do
 writeUUIDFile :: RemoteConfig -> UUID -> Annex ()
 writeUUIDFile c u = do
 	conn <- s3ConnectionRequired c u
-	go conn =<< liftIO (tryNonAsync $ getObject conn $ mkobject L.empty)
+	v <- checkUUIDFile  c u conn
+	case v of
+		Left e -> error e
+		Right True -> return ()
+		Right False -> do
+			let object = setStorageClass (getStorageClass c) (mkobject uuidb)
+			either s3Error return =<< liftIO (sendObject conn object)
   where
-	go _conn (Right (Right o)) = unless (obj_data o == uuidb) $
-		error $ "This bucket is already in use by a different S3 special remote, with UUID: " ++ L.unpack (obj_data o)
-	go conn _ = do
-		let object = setStorageClass (getStorageClass c) (mkobject uuidb)
-		either s3Error return =<< liftIO (sendObject conn object)
-
-	file = filePrefix c ++ "annex-uuid"
-	uuidb = L.pack $ fromUUID u
+	file = uuidFile c
+	uuidb = L.fromChunks [T.encodeUtf8 $ T.pack $ fromUUID u]
 	bucket = fromJust $ getBucket c
 
 	mkobject = S3Object bucket file "" (getXheaders c)
 
+{- Checks if the UUID file exists in the bucket and has the specified UUID already. -}
+checkUUIDFile :: RemoteConfig -> UUID -> AWSConnection -> Annex (Either String Bool)
+checkUUIDFile c u conn = check <$> liftIO (tryNonAsync $ getObject conn $ mkobject L.empty)
+  where
+	check (Right (Right o))
+		| obj_data o == uuidb = Right True
+		| otherwise = Left $ "This bucket is already in use by a different S3 special remote, with UUID: " ++ show (obj_data o)
+	check _ = Right False
+
+	uuidb = L.fromChunks [T.encodeUtf8 $ T.pack $ fromUUID u]
+	bucket = fromJust $ getBucket c
+	file = uuidFile c
+
+	mkobject = S3Object bucket file "" (getXheaders c)
+
+uuidFile :: RemoteConfig -> FilePath
+uuidFile c = filePrefix c ++ "annex-uuid"
+
 s3ConnectionRequired :: RemoteConfig -> UUID -> Annex AWSConnection
 s3ConnectionRequired c u =
 	maybe (error "Cannot connect to S3") return =<< s3Connection c u
