@@ -10,18 +10,14 @@
 module Types.Backend where
 
 import Types.Key
-
-{- The source used to generate a key. The location of the content
- - may be different from the filename associated with the key. -}
-data KeySource = KeySource
-	{ keyFilename :: FilePath
-	, contentLocation :: FilePath
-	}
+import Types.KeySource
 
 data BackendA a = Backend
 	{ name :: String
 	, getKey :: KeySource -> a (Maybe Key) 
 	, fsckKey :: Maybe (Key -> FilePath -> a Bool)
+	, canUpgradeKey :: Maybe (Key -> Bool)
+	, fastMigrate :: Maybe (Key -> BackendA a -> Maybe Key)
 	}
 
 instance Show (BackendA a) where
