@@ -2,7 +2,7 @@
  -
  - Copyright 2010-2013 Joey Hess <joey@kitenet.net>
  -
- - Licensed under the GNU GPL version 3 or higher.
+ - License: BSD-2-clause
  -}
 
 {-# LANGUAGE CPP #-}
@@ -36,7 +36,7 @@ copyFileExternal src dest = do
 {- Create a hard link if the filesystem allows it, and fall back to copying
  - the file. -}
 createLinkOrCopy :: FilePath -> FilePath -> IO Bool
-#ifndef __WINDOWS__
+#ifndef mingw32_HOST_OS
 createLinkOrCopy src dest = go `catchIO` const fallback
   where
   	go = do
