@@ -2,12 +2,15 @@
  -
  - Copyright 2012-2013 Joey Hess <joey@kitenet.net>
  -
- - Licensed under the GNU GPL version 3 or higher.
+ - License: BSD-2-clause
  -}
 
 module Utility.HumanTime (
 	Duration(..),
+	durationSince,
 	durationToPOSIXTime,
+	durationToDays,
+	daysToDuration,
 	parseDuration,
 	fromDuration,
 	prop_duration_roundtrips
@@ -17,6 +20,7 @@ import Utility.PartialPrelude
 import Utility.Applicative
 import Utility.QuickCheck
 
+import Data.Time.Clock
 import Data.Time.Clock.POSIX (POSIXTime)
 import Data.Char
 import Control.Applicative
@@ -25,9 +29,20 @@ import qualified Data.Map as M
 newtype Duration = Duration { durationSeconds :: Integer }
   deriving (Eq, Ord, Read, Show)
 
+durationSince :: UTCTime -> IO Duration
+durationSince pasttime = do
+	now <- getCurrentTime
+	return $ Duration $ round $ diffUTCTime now pasttime
+
 durationToPOSIXTime :: Duration -> POSIXTime
 durationToPOSIXTime = fromIntegral . durationSeconds
 
+durationToDays :: Duration -> Integer
+durationToDays d = durationSeconds d `div` dsecs
+
+daysToDuration :: Integer -> Duration
+daysToDuration i = Duration $ i * dsecs
+
 {- Parses a human-input time duration, of the form "5h", "1m", "5h1m", etc -}
 parseDuration :: String -> Maybe Duration
 parseDuration = Duration <$$> go 0
@@ -44,7 +59,7 @@ parseDuration = Duration <$$> go 0
 fromDuration :: Duration -> String
 fromDuration Duration { durationSeconds = d }
 	| d == 0 = "0s"
-	| otherwise = concat $ map showunit $ go [] units d
+	| otherwise = concatMap showunit $ go [] units d
   where
 	showunit (u, n)
 		| n > 0 = show n ++ [u]
