@@ -28,7 +28,7 @@ UPGRADE_LOCATION=http://downloads.kitenet.net/git-annex/windows/current/git-anne
 
 # Don't allow build artifact from a past successful build to be extracted
 # if we fail.
-#rm -f git-annex-installer.exe
+rm -f git-annex-installer.exe
 
 # Install haskell dependencies.
 # cabal install is not run in cygwin, because we don't want configure scripts
@@ -43,7 +43,7 @@ cabal install --only-dependencies || true
 if [ -e last-incremental-failed ]; then
 	cabal clean || true
 	# windows breakage..
-	rm -rf dist
+	rm -rf dist dist.old || mv -v dist dist.old
 fi
 touch last-incremental-failed
 
@@ -68,3 +68,7 @@ rm -f last-incremental-failed
 # (doesn't currently work well on autobuilder, reason unknown)
 rm -rf .t
 withcyg dist/build/git-annex/git-annex.exe test || true
+
+rm -f dist/build-version
+ghc --make Build/BuildVersion.hs
+Build/BuildVersion > dist/build-version
